/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file template_ns_register.c
 *   @brief This is the template file for NickServ REGISTER Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the NickServ REGISTER command.
 *
 *   The NickServ REGISTER Command allows the user to register a new user account.<br/>
 *   If EAS (Email Activation System) is enabled, you will find a passcode in you mailbox,<br/>
 *   don't answer that email - follow the Instructions there to verify your Email-Address!
 *
 *   Syntax is: /msg NickServ REGISTER < password > < email >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "global.h"

#define SERVICE "ns"
#define MODUL "register"
#define MODUL_FILE "ns_register"
#define MODUL_VERSION "1.0"

MOD_BOOL ns_register(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_register_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_register_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"register","register");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,2,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,USER);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"register",ns_register,is_user,is_user,4,5);
	addCommand(thread,&Module,"help register",ns_register_help,is_user,is_user,4,4);
	addCommand(thread,&Module,"help",ns_register_help_listing,is_user,is_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the NickServ REGISTER Command.
 * 
 * The NickServ REGISTER Command allows the user to register a new user account.<br/>
 *   If EAS (Email Activation System) is enabled, you will find a passcode in you mailbox,<br/>
 *   don't answer that email - follow the Instructions there to verify your Email-Address!
 *
 * Syntax is: /msg NickServ REGISTER < password > < email >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_register(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /* Global Variables */
	char tmppasswd[MAX_PASSWORD_LEN + 1];
	char passcode[PASSWORD_LEN];
	char From[80];
	char Subject[80];
	time_t now;
	Users *user = NULL;
	
	/* Initialise Variables to (NULL) */
	bzero(&tmppasswd,sizeof(tmppasswd));
	bzero(&passcode,sizeof(passcode));
	bzero(&From,sizeof(From));
	bzero(&Subject,sizeof(Subject));

	/*
	 * Register Command (how it may look like)
	 */
	/* Proof if the command executors GLOBAL level is high enough to use this command
	 * and the length of param[4] is lower than MAX_PASSWORD_LEN */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"register") && dynlen(param[4])<=MAX_PASSWORD_LEN){ //cmd register

	/* We Proof if "StrictPasswords" was NOT set in config (cs->,ns->,bs->,Q->,SC->,O->,m-> are config handles)
	 * AND the length of param[4] is higher the minimum password length set to nickserv config */
		if((!m->StrictPasswords && dynlen(param[4])>=ns->min_passwordlen) ||    /* OR */
		
	/* We Proof if "StrictPasswords" was set in config (cs->,ns->,bs->,Q->,SC->,O->,m-> are config handles)
	 * AND the nickname of the executor IS NOT in the password string
	 * AND the length of param[4] is higher the minimum password length set to nickserv config */
			(m->StrictPasswords && !strins(nick,param[4]) && dynlen(param[4])>=ns->min_passwordlen))
		{

			/* If parameter nr. 5 is not missing
			 * you are able to do somthing like that if you want to give
			 * detailled information about (missing/too many) parameters in the module/command itself */
			if(param[5]!='\0'){

				if(MailValidate(param[5])){ //email ok and accepted
				
				    /* If nick is suspended */
					if(thread->sql.access.settings&NS_SUSPENDED ||  /* OR */
					/* nick is forbidden */
						thread->sql.aliases.forbid)
					send_lang(thread,language,howtosend,snick,nick,"NICK_SUSPENDED_OR_FORBIDDEN",nick);

					/* If nick is logged in (thread->sql.aliases.active==1)
					 * AND registered (thread->sql.aliases.authnick[0]!='\0') */
					else if(thread->sql.aliases.active && thread->sql.aliases.authnick[0]!='\0')
						send_lang(thread,language,howtosend,snick,nick,"REGISTERED_ALREADY");
					
					/* We ask how often the same Email was used in Register (with the sql)
					 * -> If it's as often used as m->NSMaxGroups we skip registering that user and tell him */
					else if(stoi(mysql_sel_var(thread,"SELECT count(1) FROM `orion_ns_core` WHERE `email`='%s'",mysql_quote(thread,param[5])))==m->NSMaxGroups){
						send_lang(thread,language,howtosend,snick,nick,"EMAIL_USED_TOO_OFTEN_LIMIT_X",m->NSMaxGroups);
						return MOD_CONT;
					}
					
					/* If the nick we want to register is NOT already registered */
					else if(thread->sql.aliases.nick[0]=='\0'){
						
					}else send_lang(thread,language,howtosend,snick,nick,"NICK_REGISTERED_BY_AN_OTHER_USER");
					
				}else send_lang(thread,language,howtosend,snick,nick,"NS_REGISTER_INVALID_EMAIL");
				
			}else send_lang(thread,language,howtosend,snick,nick,"NS_REGISTER_FORGOT_TYPE_EMAIL");
		}else{
			send_lang(thread,language,howtosend,snick,nick,"PASSWORD_TOO_SHORT",ns->min_passwordlen);
			send_lang(thread,language,howtosend,snick,nick,"NS_REGISTER_SYNTAX",snick);
		}
	}else{
	/*
	 *  Error Messages (the ones from the main "if" to ensure we cover all cases)
	 */
	 	if(dynlen(param[4])>MAX_PASSWORD_LEN)	send_lang(thread,language,howtosend,snick,nick,"PASSWORD_TOO_LONG",MAX_PASSWORD_LEN);
		else									send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the NickServ REGISTER Command (/msg NickServ HELP REGISTER)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP REGISTER,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP REGISTER
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_register_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Register Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help register"))
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_REGISTER",snick,snick,snick);

	else send_lang(thread,language,howtosend,snick,nick,"NS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief NickServ Help List Command - REGISTER part (/msg NickServ help)
 * 
 * When the user tries to get general NickServ help, using /msg NickServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the REGISTER command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_register_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_NS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Register (Help listing)
	 */
	if(Users_NS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_CMD_REGISTER");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
