/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file cs_clear.c
 *   @brief This is the Module file for ChanServ CLEAR Command.
 *
 *   It contains all functions which are neccessary to initialize and use <br/>
 *   the ChanServ CLEAR command.
 *
 *   The ChanServ CLEAR Command allows the user to clear the channel<br/>
 *   with various CLEAR-Options. (CLEAR ops/voices/bans etc.)
 *
 *   Syntax is: /msg ChanServ CLEAR #channel OPTION
 *
 *<table border="0">
 *    <tr><td> MODES	</td><td> Syntax is: /msg ChanServ CLEAR #channel MODES		</td><td> (unsets all channel modes)			</td></tr>
 *    <tr><td> BANS		</td><td> Syntax is: /msg ChanServ CLEAR #channel BANS		</td><td> (clears all bans -> -b *)				</td></tr>
 *    <tr><td> OPS		</td><td> Syntax is: /msg ChanServ CLEAR #channel OPS		</td><td> (unsets all ops -> -o *)				</td></tr>
 *    <tr><td> HOPS		</td><td> Syntax is: /msg ChanServ CLEAR #channel HOPS		</td><td> (unsets all hops -> -h *)				</td></tr>
 *    <tr><td> VOICES	</td><td> Syntax is: /msg ChanServ CLEAR #channel VOICES	</td><td> (unsets all voices -> -v *)			</td></tr>
 *    <tr><td> USERS	</td><td> Syntax is: /msg ChanServ CLEAR #channel USERS		</td><td> (kicks all users from the channel)	</td></tr>
 *</table>
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "webform.h"
#include "global.h"

#define SERVICE "cs"
#define MODUL "clear"
#define MODUL_FILE "cs_clear"
#define MODUL_VERSION "1.2"

MOD_BOOL cs_clear_modes(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_clear_bans(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_clear_hops(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_clear_ops(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_clear_voices(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_clear_users(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL cs_clear_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_clear_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by <br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"clear * option[1]","clear");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,WEBACCESS,LOGINONLY,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addLeastModuleLevels(thread,&Module,"clear",is_services_admin,is_services_user); /* just to add a displaylevel for /msg cs help
															  this should always be the lowest used Module level! */
															  
	addCommand(thread,&Module,"clear modes",cs_clear_modes,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"clear bans",cs_clear_bans,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"clear hops",cs_clear_hops,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"clear ops",cs_clear_ops,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"clear voices",cs_clear_voices,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"clear users",cs_clear_users,is_services_admin,is_services_user,5,5);												  
	
	addCommand(thread,&Module,"help clear",cs_clear_help,is_services_user,is_services_user,4,4);
	addCommand(thread,&Module,"help",cs_clear_help_listing,is_services_user,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief CS CLEAR Modes function for the ChanServ CLEAR Command.
 * 
 * The ChanServ CLEAR Modes Command allows the user to clear all Modes in #channel!
 *  
 * Syntax is: /msg ChanServ CLEAR #channel MODES
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_clear_modes(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	char CBot[MAX_NICK_LEN];
	int sender_level = 0;
	
	/* Initialise Variables to (NULL) */
	bzero(&CBot,sizeof(CBot));
	
	/*
	 * Clear Modes Command
	 */
 	// :shock PRIVMSG chanserv :clear #channel modes
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"clear modes")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		/* Set ChanServ to our Bot, if BotServ is not loaded, or no Bot is assigned, or if FANTASY is OFF */
		CSCheckBot(CBot,thread->c.chan.bot,thread->c.chan.botserv.set,sizeof(CBot));
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){
			sender_level = stoi(thread->c.chan.access.ownlevel);
				
			if(sender_level>=thread->c.chan.levels.clear || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"clear modes") || streqnc(thread->c.chan.founder,thread->sql.aliases.authnick)){
               	strdelc(thread->c.chan.mode,'O');
               	strdelc(thread->c.chan.mode,'r');
               	do_mode(thread,CBot,param[4],thread->c.chan.datecreate,"-%s",thread->c.chan.mode);
               	send_lang(thread,language,howtosend,snick,nick,"CS_CLEAR_ALL_MODES_CLEARED",param[4]);
               	
               	setWebSuccess(thread); /* Indentify webform that changes were applied */
			}
            else send_lang(thread,language,howtosend,snick,nick,"ACCESS_IN_CHANNEL_TOO_LOW",sender_level,param[4]);
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS CLEAR Bans function for the ChanServ CLEAR Command.
 * 
 * The ChanServ CLEAR Bans Command allows the user to clear all Bans in #channel!
 *  
 * Syntax is: /msg ChanServ CLEAR #channel BANS
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_clear_bans(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	char CBot[MAX_NICK_LEN];
	int sender_level = 0;
	
	/* Initialise Variables to (NULL) */
	bzero(&CBot,sizeof(CBot));
	
	/*
	 * Clear Bans Command
	 */
 	// :shock PRIVMSG chanserv :clear #channel bans
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"clear bans")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		/* Set ChanServ to our Bot, if BotServ is not loaded, or no Bot is assigned, or if FANTASY is OFF */
		CSCheckBot(CBot,thread->c.chan.bot,thread->c.chan.botserv.set,sizeof(CBot));
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){
			sender_level = stoi(thread->c.chan.access.ownlevel);
				
			if((sender_level>=thread->c.chan.levels.clear/* && sender_level>=SingleLevel(thread,"unban",thread->c.chan.name)*/) || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"clear bans") || streqnc(thread->c.chan.founder,thread->sql.aliases.authnick)){
				unbanall(thread,CBot,param[4]);
               	send_lang(thread,language,howtosend,snick,nick,"CS_CLEAR_ALL_BANS_CLEARED",param[4]);
               	
               	setWebSuccess(thread); /* Indentify webform that changes were applied */
			}
            else send_lang(thread,language,howtosend,snick,nick,"ACCESS_IN_CHANNEL_TOO_LOW",sender_level,param[4]);
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS CLEAR Ops function for the ChanServ CLEAR Command.
 * 
 * The ChanServ CLEAR Ops Command allows the user to clear all Ops in #channel!
 *  
 * Syntax is: /msg ChanServ CLEAR #channel OPS
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_clear_ops(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	deepSQL users;
	char CBot[MAX_NICK_LEN];
	int sender_level = 0;
	
	/* Initialise Variables to (NULL) */
	bzero(&users,sizeof(users));
	bzero(&CBot,sizeof(CBot));
	
	/*
	 * Clear Ops Command
	 */
 	// :shock PRIVMSG chanserv :clear #channel ops
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"clear ops")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		/* Set ChanServ to our Bot, if BotServ is not loaded, or no Bot is assigned, or if FANTASY is OFF */
		CSCheckBot(CBot,thread->c.chan.bot,thread->c.chan.botserv.set,sizeof(CBot));
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){
			sender_level = stoi(thread->c.chan.access.ownlevel);
				
			if((sender_level>=thread->c.chan.levels.clear/* && sender_level>=SingleLevel(thread,"opdeop",thread->c.chan.name)*/) || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"clear ops") || streqnc(thread->c.chan.founder,thread->sql.aliases.authnick)){
				deopall(thread,CBot,param[4]);
                send_lang(thread,language,howtosend,snick,nick,"CS_CLEAR_ALL_OPS_CLEARED",param[4]);
                
                setWebSuccess(thread); /* Indentify webform that changes were applied */
			}
            else send_lang(thread,language,howtosend,snick,nick,"ACCESS_IN_CHANNEL_TOO_LOW",sender_level,param[4]);
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS CLEAR Hops function for the ChanServ CLEAR Command.
 * 
 * The ChanServ CLEAR Hops Command allows the user to clear all Hops in #channel!
 *  
 * Syntax is: /msg ChanServ CLEAR #channel HOPS
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_clear_hops(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	char CBot[MAX_NICK_LEN];
	int sender_level = 0;
	
	/* Initialise Variables to (NULL) */
	bzero(&CBot,sizeof(CBot));
	
	/*
	 * Clear Hops Command
	 */
 	// :shock PRIVMSG chanserv :clear #channel hops
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"clear hops")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		/* Set ChanServ to our Bot, if BotServ is not loaded, or no Bot is assigned, or if FANTASY is OFF */
		CSCheckBot(CBot,thread->c.chan.bot,thread->c.chan.botserv.set,sizeof(CBot));
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){
			sender_level = stoi(thread->c.chan.access.ownlevel);
				
			if((sender_level>=thread->c.chan.levels.clear/* && sender_level>=SingleLevel(thread,"dehalfop",thread->c.chan.name)*/) || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"clear hops") || streqnc(thread->c.chan.founder,thread->sql.aliases.authnick)){
				dehopall(thread,CBot,param[4]);
               	send_lang(thread,language,howtosend,snick,nick,"CS_CLEAR_ALL_HOPS_CLEARED",param[4]);
               	
               	setWebSuccess(thread); /* Indentify webform that changes were applied */
			}
            else send_lang(thread,language,howtosend,snick,nick,"ACCESS_IN_CHANNEL_TOO_LOW",sender_level,param[4]);
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS CLEAR Voices function for the ChanServ CLEAR Command.
 * 
 * The ChanServ CLEAR Voices Command allows the user to clear all Voices in #channel!
 *  
 * Syntax is: /msg ChanServ CLEAR #channel VOICES
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_clear_voices(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	char CBot[MAX_NICK_LEN];
	int sender_level = 0;
	
	/* Initialise Variables to (NULL) */
	bzero(&CBot,sizeof(CBot));
	
	/*
	 * Clear Voices Command
	 */
 	// :shock PRIVMSG chanserv :clear #channel voices
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"clear voices")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		/* Set ChanServ to our Bot, if BotServ is not loaded, or no Bot is assigned, or if FANTASY is OFF */
		CSCheckBot(CBot,thread->c.chan.bot,thread->c.chan.botserv.set,sizeof(CBot));
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){
			sender_level = stoi(thread->c.chan.access.ownlevel);
				
			if((sender_level>=thread->c.chan.levels.clear/* && sender_level>=SingleLevel(thread,"devoice",thread->c.chan.name)*/) || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"clear voices") || streqnc(thread->c.chan.founder,thread->sql.aliases.authnick)){
				devoiceall(thread,CBot,param[4]);
                send_lang(thread,language,howtosend,snick,nick,"CS_CLEAR_ALL_VOICES_CLEARED",param[4]);
                
                setWebSuccess(thread); /* Indentify webform that changes were applied */
			}
            else send_lang(thread,language,howtosend,snick,nick,"ACCESS_IN_CHANNEL_TOO_LOW",sender_level,param[4]);
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS CLEAR Users function for the ChanServ CLEAR Command.
 * 
 * The ChanServ CLEAR Users Command allows the user to clear all Users in #channel!
 *  
 * Syntax is: /msg ChanServ CLEAR #channel USERS
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_clear_users(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	deepSQL users;
	char CBot[MAX_NICK_LEN];
	int sender_level = 0;
	int i = 0;
	
	/* Initialise Variables to (NULL) */
	bzero(&users,sizeof(users));
	bzero(&CBot,sizeof(CBot));
	
	/*
	 * Clear Users Command
	 */
 	// :shock PRIVMSG chanserv :clear #channel users
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"clear users")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		/* Set ChanServ to our Bot, if BotServ is not loaded, or no Bot is assigned, or if FANTASY is OFF */
		CSCheckBot(CBot,thread->c.chan.bot,thread->c.chan.botserv.set,sizeof(CBot));
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){
			sender_level = stoi(thread->c.chan.access.ownlevel);
				
			if((sender_level>=thread->c.chan.levels.clear/* && sender_level>=SingleLevel(thread,"deop",thread->c.chan.name)*/) || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"clear users") || streqnc(thread->c.chan.founder,thread->sql.aliases.authnick)){
    			users.res    = doSelectMySQL(thread,"SELECT `nick` FROM `orion_chan_users` WHERE `channel`='%s'",mysql_quote(thread,param[4]));
        		users.fields = mysql_num_rows(users.res);
    			i = 0;
    				
    			do_mode(thread,CBot,param[4],thread->c.chan.datecreate,"+%c",IRC.InviteCMode);
                while((users.row = mysql_fetch_row(users.res))){
					if(!streqnc(users.row[0],nick))
    					do_kick(thread,CBot,param[4],users.row[0],"resolved clear (users) which caused a kick.");
    				i++;
    			}
                send_lang(thread,language,howtosend,snick,nick,"CS_CLEAR_ALL_USERS_CLEARED");
                	
                mysql_free_result_myr(users.res); //free MySQL result
                
                setWebSuccess(thread); /* Indentify webform that changes were applied */
			}else send_lang(thread,language,howtosend,snick,nick,"ACCESS_IN_CHANNEL_TOO_LOW",sender_level,param[4]);
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the ChanServ CLEAR Command (/msg ChanServ HELP CLEAR)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP CLEAR,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP CLEAR
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_clear_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Clear Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help clear")){
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_CLEAR");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}

	else send_lang(thread,language,howtosend,snick,nick,"CS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief ChanServ Help List Command - CLEAR part (/msg ChanServ help)
 * 
 * When the user tries to get general ChanServ help, using /msg ChanServ help, all loaded commands with <br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the CLEAR command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_clear_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_CS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Clear (Help listing)
	 */
	if(Users_CS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_CMD_CLEAR");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
