/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file cs_list.c
 *   @brief This is the Module file for ChanServ LIST Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the ChanServ LIST command.
 *
 *   The ChanServ LIST Command allows the user to dump all<br/>
 *   registered channels matching a given pattern.<br/>
 *   (Channels with the PRIVATE option set are not listed. - except for ServicesAdmins)<br/>
 *   NOTE: This Command may be limited to ServicesAdmins (by the services.conf)
 *
 *   Syntax is: /msg ChanServ LIST [*channelpattern*]
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"

#define SERVICE "cs"
#define MODUL "list"
#define MODUL_FILE "cs_list"
#define MODUL_VERSION "1.2"

MOD_BOOL cs_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_list_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_list_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"list","list");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);

	if(m->CSListOpersOnly){
		addOptions(thread,&Module,4,WEBACCESS,LOGINONLY,OPER_IF_DEFCON,OPERONLY);
		addAccess (thread,&Module,ADMIN);
	}else{
		addOptions(thread,&Module,3,WEBACCESS,LOGINONLY,OPER_IF_DEFCON);
		addAccess (thread,&Module,BOTH);
	}
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"list",cs_list,is_services_admin,is_services_user,4,4);
	addCommand(thread,&Module,"help list",cs_list_help,is_services_admin,is_services_user,4,4);
	addCommand(thread,&Module,"help",cs_list_help_listing,is_services_admin,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the ChanServ LIST Command.
 * 
 * The ChanServ LIST Command allows the user to dump all<br/>
 *   registered channels matching a given pattern.<br/>
 *   (Channels with the PRIVATE option set are not listed. - except for ServicesAdmins)
 *
 * Syntax is: /msg ChanServ LIST [*channelpattern*]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	Channel c;
	deepSQL chans;
	char pattern[BUFSIZE];
	int i=5, suspended=0, forbidden=0, noexpire=0;

	/* Initialise Variables to (NULL) */
	bzero(&c,sizeof(c));
	bzero(&chans,sizeof(chans));
	bzero(&pattern,sizeof(pattern));

	/*
	 * List Command
	 */
	if(strcor(param[4],"*") && thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"list")){
		if(thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"list")){
			while(param[i]){
				if(streqnc(param[i],"forbidden")) forbidden=1;
				else if(streqnc(param[i],"suspend")) suspended=1;
				else if(streqnc(param[i],"noexpire")) noexpire=1;
				i++;
			}
		}
		
		if(param[4]){
			dynncpy(pattern,param[4],sizeof(pattern));
			strrepc(pattern,'*','%');
			chans.res = doSelectMySQL(thread,"SELECT `channel` FROM `orion_cs_core` WHERE `serv`='ChanServ' AND `channel` LIKE '%s' ORDER BY `channel` ASC LIMIT %d",pattern,m->CSListMax);
		}
		else
			chans.res = doSelectMySQL(thread,"SELECT `channel` FROM `orion_cs_core` WHERE `serv`='ChanServ' ORDER BY `channel` ASC LIMIT %d",m->CSListMax);
		chans.fields  = mysql_num_rows(chans.res);

		//strrepc(param[4],'%','*');
		if(chans.fields)
			send_lang(thread,language,howtosend,snick,nick,"CS_LIST_HEADER",param[4]);
		
		i=0;
		for(i=0;i<chans.fields;i++){
			chans.row = mysql_fetch_row(chans.res);
			
			bzero(&c,sizeof(c));
			c = sql_chan(thread,chans.row[0]);
			
			if(c.fields && (c.settings & CS_SUSPEND_BIT) && suspended==1)														send_lang(thread,language,howtosend,snick,nick,"CS_LIST_LIST_SUSPENDED",chans.row[0],c.reason);
			else if(c.fields && (c.settings & CS_FORBID_BIT) && forbidden==1)													send_lang(thread,language,howtosend,snick,nick,"CS_LIST_LIST_FORBIDDEN",chans.row[0],c.reason);
			else if(c.settings&CS_PRIVATE_BIT && thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"list"))	send_lang(thread,language,howtosend,snick,nick,"CS_LIST_LIST_PRIVATE"  ,chans.row[0],c.description);
			else if(c.fields && !c.expire && noexpire==1)																		send_lang(thread,language,howtosend,snick,nick,"CS_LIST_LIST_NOEXPIRE" ,chans.row[0],c.description);
			else																												send_lang(thread,language,howtosend,snick,nick,"CS_LIST_LIST"          ,chans.row[0],c.description);
		}
		if(chans.fields)
			send_lang(thread,language,howtosend,snick,nick,"CS_LIST_FOOTER",i,chans.fields);
		else
			send_lang(thread,language,howtosend,snick,nick,"CS_LIST_NO_ADDED_CHANS");
			
		mysql_free_result_myr(chans.res);
		
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}else{
	/*
	 *  Error Messages
	 */
		if(!strcor(param[4],"*"))																send_lang(thread,language,howtosend,snick,nick,"WRONG_INPUT_PATTERN_INPUT",param[4]);
		else if(thread->sql.access.rights<ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"list"))	send_lang(thread,language,howtosend,snick,nick,"LEVEL_NOT_HIGH_ENOUGH");
		else																					send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the ChanServ LIST Command (/msg ChanServ HELP LIST)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP LIST,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP LIST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_list_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * List Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help list")){
		send_lang(thread,language,howtosend,snick,nick,"CS_SERVADMIN_HELP_LIST",snick,snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}

	else send_lang(thread,language,howtosend,snick,nick,"CS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief ChanServ Help List Command - LIST part (/msg ChanServ help)
 * 
 * When the user tries to get general ChanServ help, using /msg ChanServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the LIST command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_list_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_CS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * List (Showcommands)
	 */
	if(Users_CS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_CMD_LIST");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
