/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Chrisostomos Adamos (alias: Ally), Email: Ally@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file hs_on.c
 *   @brief This is the Module file for HostServ ON Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the HostServ ON command.
 *
 *   The HostServ ON Command allows the user to activate the vHost<br/>
 *   currently assigned to the nick in use.
 *
 *   Syntax is: /msg HostServ ON
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"

#define SERVICE "hs"
#define MODUL "on"
#define MODUL_FILE "hs_on"
#define MODUL_VERSION "1.2"

MOD_BOOL hs_on(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL hs_on_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL hs_on_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module;
	
	if(!IRC.VHost){
		Module.ModErr = MOD_ERR_CANCELLED;
		Module.module = NULL;
		return Module;
	}
	
	Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"on","on");

	addAuthor (thread,&Module,"Ally, cLiEbiG, (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,WEBACCESS,LOGINONLY,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"on",hs_on,is_services_user,is_services_user,3,3);
	addCommand(thread,&Module,"help on",hs_on_help,is_services_user,is_services_user,4,4);
	addCommand(thread,&Module,"help",hs_on_help_listing,is_services_user,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the HostServ ON Command.
 * 
 * The HostServ ON Command allows the user to activate the vHost<br/>
 *   currently assigned to the nick in use.
 *
 * Syntax is: /msg HostServ ON
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL hs_on(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
    deepSQL checkvhost;
	char *HostParts[2];
	char* vIdent = NULL;
	char* vHost  = NULL;
	char virthost[MAX_HOST_LEN + MAX_IDENT_LEN + 2];
	size_t parts = 0;
	
	/* Initialise Variables to (NULL) */
	bzero(&checkvhost,sizeof(checkvhost));
	bzero(HostParts,sizeof(HostParts));
	bzero(&virthost,sizeof(virthost));

	/*
	 * On Command
	 */
   	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"on")){

		checkvhost.res    = doSelectMySQL(thread,"SELECT `nick`, `active`, `vhost` FROM `orion_hs_core` WHERE `nick`='%s'",mysql_quote(thread,nick));
		checkvhost.row    = mysql_fetch_row(checkvhost.res);
		checkvhost.fields = mysql_num_rows(checkvhost.res);
		
		if(checkvhost.fields){
			if(stoi(checkvhost.row[1]) == 0){
				domysql(thread,"UPDATE `orion_hs_core` SET `active` = '1' WHERE `nick`='%s'",mysql_quote(thread,nick));

				dynncpy(virthost,checkvhost.row[2],sizeof(virthost));
				parts = strsplit(virthost,HostParts,'@');
				
                if(parts==2){ /* vIdent + vHost */
					vIdent = HostParts[0];
					vHost = HostParts[1];
					
        			do_chgident(thread,snick,nick,vIdent);
        			do_chghost(thread,snick,nick,vHost);
					send_lang(thread,language,howtosend,snick,nick,"HS_ON_SUCCESS",_dynadd("%s@%s",vIdent,vHost),snick);
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}else if(parts==1){  /* nur vHost */
					vHost = HostParts[0];
					
					do_chghost(thread,snick,nick,vHost);
					send_lang(thread,language,howtosend,snick,nick,"HS_ON_SUCCESS",vHost,snick);
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}
			}else send_lang(thread,language,howtosend,snick,nick,"HS_ON_IS_ACTIVE",snick);
		}else send_lang(thread,language,howtosend,snick,nick,"HS_DEL_NO_VHOST",nick);
		
		/* mysql_free_result_myr(); - checkvhost */
		mysql_free_result_myr(checkvhost.res);
    }else{
	/*
	 * Error Messages
	 */
 		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
    }
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the HostServ ON Command (/msg HostServ HELP ON)
 * 
 * When the user tries to get help for this command, using /msg HostServ HELP ON,<br/>
 * this function is executed.
 * 
 * Syntax: /msg HostServ HELP ON
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL hs_on_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * On Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help on")){
		send_lang(thread,language,howtosend,snick,nick,"HS_HELP_ON",snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"HS_NO_HELP_AVAIL",snick);	
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief HostServ Help List Command - ON part (/msg HostServ help)
 * 
 * When the user tries to get general HostServ help, using /msg HostServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the ON command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL hs_on_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_HS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * On (Help Listing)
	 */
	if(Users_HS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"HS_HELP_CMD_ON");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
