/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Chrisostomos Adamos (alias: Ally), Email: Ally@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 371                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-12-02 00:51:13 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file hs_request.c
 *   @brief This is the Module file for HostServ REQUEST Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the HostServ REQUEST command.
 *
 *   The HostServ REQUEST Command allows the user to request the given vHost<br/>
 *   to be actived for his nick by the network administrators.<br/>
 *   Please be patient while the request is being considered. (may take some time)
 *
 *   Syntax is: /msg HostServ REQUEST < vhost >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"

#define SERVICE "hs"
#define MODUL "request"
#define MODUL_FILE "hs_request"
#define MODUL_VERSION "1.2"

MOD_BOOL hs_request(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL hs_request_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL hs_request_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module;
	
	if(!IRC.VHost){
		Module.ModErr = MOD_ERR_CANCELLED;
		Module.module = NULL;
		return Module;
	}
	
	Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"request","request");

	addAuthor (thread,&Module,"Ally, cLiEbiG, (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 371 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,WEBACCESS,LOGINONLY,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"request",hs_request,is_services_user,is_services_user,4,4);
	addCommand(thread,&Module,"help request",hs_request_help,is_services_user,is_services_user,4,4);
	addCommand(thread,&Module,"help",hs_request_help_listing,is_services_user,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the HostServ REQUEST Command.
 * 
 * The HostServ REQUEST Command allows the user to request the given vHost<br/>
 *   to be actived for his nick by the network administrators.<br/>
 *   Please be patient while the request is being considered. (may take some time)
 *
 * Syntax is: /msg HostServ REQUEST < vhost >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL hs_request(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
    deepSQL checkvhost;
	char *HostParts[2];
	char* vIdent = NULL;
	char* vHost  = NULL;
	char tempnick[MAX_IDENT_LEN+MAX_HOST_LEN+1];
	char virthost[MAX_HOST_LEN];
	size_t parts = 0;
	bool ret = false;
	
	/* Initialise Variables to (NULL) */
	bzero(&checkvhost,sizeof(checkvhost));
	bzero(HostParts,sizeof(HostParts));
	bzero(&tempnick,sizeof(tempnick));
	bzero(&virthost,sizeof(virthost));
	
	/*
	 * Request Command
	 */
   	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"request")){
		if(thread->sql.aliases.linked){ printf("temporary"); } // temporrer alias!
		else{

			checkvhost.res    = doSelectMySQL(thread,"SELECT `nick`,`display`,`vhost`,`changetime`,`active` FROM `orion_hs_core` WHERE `nick`='%s'",mysql_quote(thread,nick));
			checkvhost.fields = mysql_num_rows(checkvhost.res);
			checkvhost.row    = mysql_fetch_row(checkvhost.res);
			
			dynncpy(virthost,param[4],sizeof(virthost));
			int position = getposition(virthost,'@');

			if(position>-1){
                vHost = &virthost[position+1];
                virthost[position] = '\0';
                vIdent = virthost;
                parts = 2;
			}else if(position == -2){
                vHost = virthost;
                parts = 1;
			}else if(position == -1){
				parts = 0;
				printf("error!\n");
			}

			if(checkvhost.fields && streqnc(checkvhost.row[0],nick)){
				if(((time_t)stol(checkvhost.row[3]) + (expire(m->HSWaitRequest)-time(NULL))) > time(NULL)){
					// Zeit noch nicht abgelaufen zum requesten
					//dlog(thread,2,"Zeit noch nicht abgelaufen fr neuen Request\n");
					send_lang(thread,language,howtosend,snick,nick,"HS_REQUEST_NOT_ALLOWED",snick);
					return MOD_CONT;
				}
				else{
					//dlog(thread,2,"Zeit abgelaufen fr neuen Request, loesche vorhandene Daten und requeste neu\n");
					domysql(thread,"DELETE FROM `orion_hs_core` WHERE `nick`='%s'",mysql_quote(thread,nick));
				}
			}

			if(vIdent && !IRC.SupportVIdents) printf("ircd does not allow vIdents");
			else if(vIdent && IRC.SupportVIdents){
				if(dynlen(vIdent)>MAX_IDENT_LEN){
					send_lang(thread,language,howtosend,snick,nick,"HS_REQUEST_VIDENT_TOO_LONG",snick);
					//printf("ident too long!\n");

					/* mysql_free_result_myr(); - checkvhost */
					mysql_free_result_myr(checkvhost.res);
					return MOD_CONT;
				}
				//dlog(thread,2,"VIDENT FOUND: %s\n",vIdent);
				ret = is_valid_chars(vIdent,"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-.");
				if(ret == 1){
					send_lang(thread,language,howtosend,snick,nick,"HS_REQUEST_VIDENT_INVALID",snick);

					/* mysql_free_result_myr(); - checkvhost */
					mysql_free_result_myr(checkvhost.res);
					return MOD_CONT;
				}
				sprintf(tempnick,"%s",vIdent);
			}
			
			if(vHost){
				if(dynlen(vHost)>MAX_HOST_LEN){
					send_lang(thread,language,howtosend,snick,nick,"HS_REQUEST_VHOST_TOO_LONG",snick);
					//printf("host too long!\n");

					/* mysql_free_result_myr(); - checkvhost */
					mysql_free_result_myr(checkvhost.res);
					return MOD_CONT;
				}
				
				//dlog(thread,2,"VHOST FOUND: %s\n",vHost);
				ret = is_valid_chars(vHost,"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-.");
				
				if(ret == 1){
					send_lang(thread,language,howtosend,snick,nick,"HS_REQUEST_VHOST_INVALID",snick);
					return MOD_CONT;
				}
				if(vIdent && parts>=2){
					bzero(&tempnick,sizeof(tempnick));
					sprintf(tempnick,"%s@%s",vIdent,vHost);
				}
				else
		            sprintf(tempnick,"%s",vHost);
			}else{
				// missing parameter...
			}
			//dlog(thread,2,"vhost: %s\n",tempnick);
			
			domysql(thread,"INSERT INTO `orion_hs_core` (`nick`,`display`,`nick_type`,`vhost`,`changetime`,`active`) VALUES ('%s','%s','%d','%s','%ld','%d')",mysql_quote(thread,nick),mysql_quote(thread,thread->sql.aliases.authnick),0,mysql_quote(thread,tempnick),time(NULL),0);
			send_lang(thread,language,howtosend,snick,nick,"HS_REQUEST_ALLOWED",snick);
			do_logchan(thread,snick,NULL,NULL,NULL,"vHost Request: %s by %s => /msg %s add %s",tempnick,nick,hs->nick,nick);
        
			/* mysql_free_result_myr(); - checkvhost */
			mysql_free_result_myr(checkvhost.res);
			
			setWebSuccess(thread); /* Indentify webform that changes were applied */
		}
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the HostServ REQUEST Command (/msg HostServ HELP REQUEST)
 * 
 * When the user tries to get help for this command, using /msg HostServ HELP REQUEST,<br/>
 * this function is executed.
 * 
 * Syntax: /msg HostServ HELP REQUEST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL hs_request_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Request Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help request")){
		send_lang(thread,language,howtosend,snick,nick,"HS_HELP_REQUEST",snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"HS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief HostServ Help List Command - REQUEST part (/msg HostServ help)
 * 
 * When the user tries to get general HostServ help, using /msg HostServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the REQUEST command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL hs_request_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_HS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Request (Help Listing)
	 */
	if(Users_HS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"HS_HELP_CMD_REQUEST");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
