/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Chrisostomos Adamos (alias: Ally), Email: Ally@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file hs_set.c
 *   @brief This is the Module file for HostServ SET Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the HostServ SET command.
 *
 *   The HostServ SET Command allows the user to set the vHost of the<br/>
 *   nick to the given hostmask.  If your IRCD supports vIdents, then using<br/>
 *   SET < nick > < ident >@< hostmask > set Idents for users as well as vHosts.
 *
 *   Syntax is: /msg HostServ SET < nick > < hostmask >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"

#define SERVICE "hs"
#define MODUL "set"
#define MODUL_FILE "hs_set"
#define MODUL_VERSION "1.2"

MOD_BOOL hs_set(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL hs_set_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL hs_set_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module;
	
	if(!IRC.VHost){
		Module.ModErr = MOD_ERR_CANCELLED;
		Module.module = NULL;
		return Module;
	}
	
	Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"set","set");

	addAuthor (thread,&Module,"Ally, cLiEbiG, (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,5,WEBACCESS,LOGINONLY,OPER_IF_DEFCON,OPERONLY,READONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"set",hs_set,is_services_oper,INFINIT,5,5);
	addCommand(thread,&Module,"help set",hs_set_help,is_services_oper,INFINIT,4,4);
	addCommand(thread,&Module,"help",hs_set_help_listing,is_services_oper,INFINIT,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the HostServ SET Command.
 * 
 * The HostServ SET Command allows the user to set the vHost of the<br/>
 *   nick to the given hostmask.  If your IRCD supports vIdents, then using<br/>
 *   SET < nick > < ident >@< hostmask > set Idents for users as well as vHosts.
 *
 * Syntax is: /msg HostServ SET < nick > < hostmask >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL hs_set(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
    deepSQL checkvhost;
    Aliases aliases5;
	char *HostParts[2];
	char* vIdent = NULL;
	char* vHost  = NULL;
	char tempnick[MAX_IDENT_LEN+MAX_HOST_LEN+1];
	char virthost[MAX_HOST_LEN];
	size_t parts = 0;
	bool ret = false;
	
	/* Initialise Variables to (NULL) */
	bzero(&checkvhost,sizeof(checkvhost));
	bzero(&aliases5,sizeof(aliases5));
	bzero(HostParts,sizeof(HostParts));
	bzero(&tempnick,sizeof(tempnick));
	bzero(&virthost,sizeof(virthost));
	
	/*
	 * Set Command
	 */
   	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set")){

		checkvhost.res    = doSelectMySQL(thread,"SELECT `nick`, `active`, `vhost` FROM `orion_hs_core` WHERE `nick`='%s'",mysql_quote(thread,param[4]));
		checkvhost.fields = mysql_num_rows(checkvhost.res);
		checkvhost.row    = mysql_fetch_row(checkvhost.res);
		
		aliases5 = sql_aliases(thread,param[4]);

		dynncpy(virthost,param[5],sizeof(virthost));
		parts = strsplit(virthost,HostParts,'@');
		
		if(parts==2){ /* vIdent + vHost */
			vIdent = HostParts[0];
			vHost = HostParts[1];
		}else if(parts==1){  /* nur vHost */
			vHost = HostParts[0];
		}

		if(vIdent && !IRC.SupportVIdents){
			printf("ircd does not allow vIdents\n");
		}else if(vIdent && IRC.SupportVIdents){
			if(dynlen(vIdent)>MAX_IDENT_LEN){
				send_lang(thread,language,howtosend,snick,nick,"HS_REQUEST_VIDENT_TOO_LONG",snick);
				//printf("ident too long!\n");

				/* mysql_free_result_myr(); - checkvhost */
				mysql_free_result_myr(checkvhost.res);
				return MOD_CONT;
			}
			//printf("VIDENT FOUND: %s\n",vIdent);
			ret = is_valid_chars(vIdent,"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-.");
			
			if(ret == 1){
				send_lang(thread,language,howtosend,snick,nick,"HS_REQUEST_VIDENT_INVALID",snick);

				/* mysql_free_result_myr(); - checkvhost */
				mysql_free_result_myr(checkvhost.res);
				return MOD_CONT;
			}
			snprintf(tempnick,sizeof(tempnick),"%s",vIdent);
		}
		
		if(vHost){
			if(dynlen(vHost)>MAX_HOST_LEN){
				send_lang(thread,language,howtosend,snick,nick,"HS_REQUEST_VHOST_TOO_LONG",snick);
				//printf("host too long!\n");

				/* mysql_free_result_myr(); - checkvhost */
				mysql_free_result_myr(checkvhost.res);
				return MOD_CONT;
			}
			//printf("VHOST FOUND: %s\n",vHost);
			ret = is_valid_chars(vHost,"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-.");
			
			if(ret == 1){
				send_lang(thread,language,howtosend,snick,nick,"HS_REQUEST_VHOST_INVALID",snick);

				/* mysql_free_result_myr(); - checkvhost */
				mysql_free_result_myr(checkvhost.res);
				return MOD_CONT;
			}
			
			if(vIdent)
				snprintf(tempnick,sizeof(tempnick),"%s%s%s",tempnick,"@",vHost);
			else
				snprintf(tempnick,sizeof(tempnick),"%s",vHost);

		}

		if(checkvhost.fields)
			domysql(thread,"UPDATE `orion_hs_core` SET `vhost` = '%s', `changetime`='%d', `authorized`='%s', `active`='1' WHERE `nick`='%s'",mysql_quote(thread,tempnick),time(NULL),mysql_quote(thread,thread->sql.aliases.authnick),mysql_quote(thread,param[4]));
		else if(aliases5.authnick[0]!='\0')
			domysql(thread,"INSERT INTO `orion_hs_core` (`nick`,`display`,`nick_type`,`vhost`,`authorized`,`changetime`,`active`) VALUES ('%s','%s','%d','%s','%s','%d','%d')",mysql_quote(thread,param[4]),mysql_quote(thread,aliases5.authnick),0,mysql_quote(thread,tempnick),mysql_quote(thread,thread->sql.aliases.authnick),time(NULL),1);

		if(stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_core_condata` WHERE `nick`='%s'",mysql_quote(thread,param[4])))){
			if(parts==2){ /* vIdent + vHost */
				do_chgident(thread,snick,param[4],vIdent);
				do_chghost(thread,snick,param[4],vHost);
				
				send_lang(thread,language,howtosend,snick,nick,"HS_SET_SUCCESS",tempnick,param[4]);
				
				setWebSuccess(thread); /* Indentify webform that changes were applied */
			}else if(parts==1){  /* nur vHost */
				do_chghost(thread,snick,param[4],vHost);
				
				send_lang(thread,language,howtosend,snick,nick,"HS_SET_SUCCESS",tempnick,param[4]);
				
				setWebSuccess(thread); /* Indentify webform that changes were applied */
			}else
				send_lang(thread,language,howtosend,snick,nick,"HS_SET_ERROR",tempnick,param[4]);
		}
		
		/* mysql_free_result_myr(); - checkvhost */
		mysql_free_result_myr(checkvhost.res);
	}else{
	/*
	 * Error Messages
	 */
 		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the HostServ SET Command (/msg HostServ HELP SET)
 * 
 * When the user tries to get help for this command, using /msg HostServ HELP SET,<br/>
 * this function is executed.
 * 
 * Syntax: /msg HostServ HELP SET
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL hs_set_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set")){
		send_lang(thread,language,howtosend,snick,nick,"HS_HELP_SET",snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"HS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief HostServ Help List Command - SET part (/msg HostServ help)
 * 
 * When the user tries to get general HostServ help, using /msg HostServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the SET command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL hs_set_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_HS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Set (Help Listing)
	 */
	if(Users_HS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"HS_HELP_CMD_SET");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
