/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ns_alist.c
 *   @brief This is the Module file for NickServ ALIST Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the NickServ ALIST command.
 *
 *   The NickServ ALIST Command allows the user to dump all channels he/she has access to.<br/>
 *   Optionally, you can specify a level in XOP or ACCESS format. The resulting list will only<br/>
 *   include channels where you have the given level of access (Q and Chanserv Channels).<br/>
 *   Format: i.e ALIST OPDEOP | ALIST ao | ALIST o
 *
 *   Syntax is: /msg NickServ ALIST < level >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "webform.h"
#include "global.h"

#define SERVICE "ns"
#define MODUL "alist"
#define MODUL_FILE "ns_alist"
#define MODUL_VERSION "1.2"

MOD_BOOL ns_alist(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_alist_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_alist_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"alist","alist");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,WEBACCESS,LOGINONLY,OPER_IF_DEFCON);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"alist",ns_alist,is_services_admin,is_services_user,4,5);
	addCommand(thread,&Module,"help alist",ns_alist_help,is_services_admin,is_services_user,4,4);
	addCommand(thread,&Module,"help",ns_alist_help_listing,is_services_admin,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the NickServ ALIST Command.
 * 
 * The NickServ ALIST Command allows the user to dump all channels he/she has access to.<br/>
 *   Optionally, you can specify a level in XOP or ACCESS format. The resulting list will only<br/>
 *   include channels where you have the given level of access (Q and Chanserv Channels).<br/>
 *   Format: i.e ALIST OPDEOP | ALIST ao | ALIST o
 *
 * Syntax is: /msg NickServ ALIST < level >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_alist(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL chanaccess;
    Aliases aliases5;
    LevelChanging levels;
	char escaped[MAX_HOST_LEN*2];
	char query  [SQL_BUF];
	int i=0;

	/* Initialise Variables to (NULL) */
	bzero(&chanaccess,sizeof(chanaccess));
	bzero(&aliases5,sizeof(aliases5));
	bzero(&aliases5,sizeof(aliases5));
	bzero(&escaped,sizeof(escaped));
	bzero(&query,sizeof(query));
	
	/*
	 * Alist Command
	 */
 	if(!param[5]) aliases5=sql_aliases(thread,nick); //thread->sql.aliases.authnick (Q-authnick - referencenick)
 	else aliases5=sql_aliases(thread,param[4]); //thread->sql.aliases.authnick (Q-authnick - referencenick)
 	
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"alist")){
		if(param[5])
			levels = GetLevelvalueByIndex(leveltable,param[5]);
		else
			levels = GetLevelvalueByIndex(leveltable,param[4]);
		
		if(levels.index[0]!='\0'){
			snprintf(query,sizeof(query),"SELECT `orion_cs_core`.`channel`,`orion_cs_access`.`authnick`,`orion_cs_access`.`level`,`orion_cs_core`.`serv` FROM `orion_cs_core` LEFT OUTER JOIN `orion_cs_access` ON (`orion_cs_access`.`channel` = `orion_cs_core`.`channel` AND `orion_cs_access`.`authnick`='%s') WHERE",mysql_quote(thread,aliases5.authnick));
			
		    if(levels.type==3 || levels.type==4) 
				straddns(query,_dynadd(" (`orion_cs_access`.`level` >= %d OR `orion_cs_access`.`level` RLIKE '(n)')",ACCESS_FOUNDER),sizeof(query));
			else 
				straddns(query,_dynadd(" (`orion_cs_access`.`level` >= %d OR `orion_cs_access`.`level` >= %d OR `orion_cs_access`.`level` RLIKE '(n|m)')",ACCESS_FOUNDER,ACCESS_SUCCESSOR),sizeof(query));
			
		    if(levels.CSlevel[0] != '\0') 	straddns(query,_dynadd(" OR (`orion_cs_access`.`level` >= `orion_cs_core`.`%s`)",mysql_quote(thread,levels.CSlevel)),sizeof(query));
			if(levels.XOPlevel != 0) 		straddns(query,_dynadd(" OR (`orion_cs_access`.`level` >= %d AND `orion_cs_core`.`settings`&0x01)",levels.XOPlevel),sizeof(query));
			if(levels.Qlevel[0] != '\0') 	straddns(query,_dynadd(" OR (`orion_cs_access`.`level` RLIKE '(%s)')",mysql_quote(thread,levels.Qlevel)),sizeof(query));
		
	        chanaccess.res    = doSelectMySQL(thread,query);
			chanaccess.fields = mysql_num_rows(chanaccess.res);
			
			if(chanaccess.fields) {
				send_lang(thread,language,howtosend,snick,nick,"NS_ALIST_SAY_CHANNELS",aliases5.nick);
				for(i=0;i<chanaccess.fields;i++){
					chanaccess.row = mysql_fetch_row(chanaccess.res);
					
					if(chanaccess.row)
						send_lang(thread,language,howtosend,snick,nick,"NS_ALIST_ACCESS_LISTINGS",i+1,chanaccess.row[1],chanaccess.row[3],chanaccess.row[0],chanaccess.row[2]);
				}
				send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
			}else
				send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
			
	        mysql_free_result_myr(chanaccess.res); //free result
		}
		else
			send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
			
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the NickServ ALIST Command (/msg NickServ HELP ALIST)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP ALIST,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP ALIST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_alist_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Alist Help
	 */
	if((strcor(thread->sql.Cdata.modes,"o") || thread->Webform) && thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"help alist"))
		send_lang(thread,language,howtosend,snick,nick,"NS_SERVADMIN_HELP_ALIST",cs->nick);
	else if(thread->sql.access.rights>=ModuleFile_UserCommandLevel(thread,MODUL_FILE,"help alist"))
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_ALIST",cs->nick);

	else send_lang(thread,language,howtosend,snick,nick,"NS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief NickServ Help List Command - ALIST part (/msg NickServ help)
 * 
 * When the user tries to get general NickServ help, using /msg NickServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the ALIST command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_alist_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_NS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Alist (Help listing)
	 */
	if(Users_NS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_CMD_ALIST");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
