/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file o_sqline.c
 *   @brief This is the Module file for O SQLINE Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the O SQLINE command.
 *
 *   The O SQLINE Command allows an admin to prevent<br/>
 *   wthe given nick from connecting the Network.
 *
 *   Syntax is: /msg O SQLINE { ADD | DEL | LIST | CLEAR } [nick [< duration (2d) > < reason >]]
 *
 *<table border="0">
 *    <tr><td> ADD		</td><td> Syntax is: /msg O SQLINE ADD		</td><td> < nick > < duration(i.e. 2d) > < reason >	</td></tr>
 *    <tr><td> DEL		</td><td> Syntax is: /msg O SQLINE DEL		</td><td> < nick >									</td></tr>
 *    <tr><td> LIST		</td><td> Syntax is: /msg O SQLINE LIST		</td><td>											</td></tr>
 *    <tr><td> CLEAR	</td><td> Syntax is: /msg O SQLINE CLEAR	</td><td>											</td></tr>
 *</table>
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"

#define SERVICE "O"
#define MODUL "sqline"
#define MODUL_FILE "o_sqline"
#define MODUL_VERSION "1.2"

MOD_BOOL O_sqline_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_sqline_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_sqline_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_sqline_clear(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL O_sqline(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_sqline_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_sqline_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module;
	
	if(!IRC.SupportSQlines){
		Module.ModErr = MOD_ERR_CANCELLED;
		Module.module = NULL;
		return Module;
	}
	
	Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"sqline option[1]","sqline option[1]");
	
	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,5,WEBACCESS,LOGINONLY,OPER_IF_DEFCON,OPERONLY,READONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"sqline add",O_sqline_add,700,INFINIT,6,0);
	addCommand(thread,&Module,"sqline del",O_sqline_del,700,INFINIT,4,7);	
	addCommand(thread,&Module,"sqline list",O_sqline_list,700,INFINIT,4,4);
	addCommand(thread,&Module,"sqline clear",O_sqline_clear,700,INFINIT,4,4);

	addCommand(thread,&Module,"sqline",NULL,700,INFINIT,4,0);
	addCommand(thread,&Module,"help sqline",O_sqline_help,700,INFINIT,4,5);
	addCommand(thread,&Module,"help",O_sqline_help_listing,700,INFINIT,3,3); //100 - each Admin is able to see it.

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief O SQLINE Add function for the O SQLINE Command.
 * 
 * The O SQLINE Add Command allows an admin to<br/>
 *   add an Sqline for target nick
 *  
 * Syntax is: /msg O SQLINE ADD < nick > < duration(i.e. 2d) > < reason >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
 //:shock PRIVMSG OperServ :SQLINE ADD nick EXPIRE reason
MOD_BOOL O_sqline_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
		//NULL
		
	/* Initialise Variables to (NULL) */
		//NULL
		
	/*
	 * Sqline Add Command
	 */
	 //:shock PRIVMSG operserv@services.deepirc.net :sqline add 30d _Babygirl_ fuck you =D (take the baseball racket ^^)
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"sqline add") && param[7]){
		if(check_expire(param[6])){
			if(IRC.SupportChanSQlines){
            	if(is_valid_chars(param[5],"#0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-_*")){
                	send_lang(thread,language,howtosend,snick,nick,"INVALID_PARAMETER",param[5]);
            	    return MOD_CONT;
				}
			}else{
            	if(is_valid_chars(param[5],"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-_*")){
                	send_lang(thread,language,howtosend,snick,nick,"INVALID_PARAMETER",param[5]);
            	    return MOD_CONT;
				}
			}
			do_sqline(thread,_dynadd("%s!%s@%s",O->nick,O->ident,O->host),param[5],expire(param[6]) == 0 ? 0 : expire(param[6])+time(NULL),time(NULL),_dstrjoin(' ',param,7));
			send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
			
			setWebSuccess(thread); /* Indentify webform that changes were applied */
		}else
			send_lang(thread,language,howtosend,snick,nick,"INVALID_EXPIRESTRING",param[6]);
	}else{
	/*
	 *  Error Messages
	 */
		if(!param[7])	send_lang(thread,language,howtosend,snick,nick,"NO_REASON");
		else 			send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O SQLINE Del function for the O SQLINE Command.
 * 
 * The O SQLINE Del Command allows an admin to<br/>
 *   delete an Sqline for target nick
 *  
 * Syntax is: /msg O SQLINE DEL < nick >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL O_sqline_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
		//NULL
		
	/* Initialise Variables to (NULL) */
		//NULL
		
	/*
	 * Sqline Del Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"sqline del")){
		do_unsqline(thread,_dynadd("%s!%s@%s",O->nick,O->ident,O->host),m->servername,param[5]);
		send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
		
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O SQLINE List function for the O SQLINE Command.
 * 
 * The O SQLINE List Command allows an admin to<br/>
 *   dump the Sqline-List
 *  
 * Syntax is: /msg O SQLINE LIST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL O_sqline_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL sqline;
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&sqline,sizeof(sqline));

	/*
	 * Sqline List Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"sqline list")){
		sqline.res    = doSelectMySQL(thread,"SELECT `qline`,`duration`,`addedby`,`reason` FROM `orion_os_sqlines`");
		sqline.fields = mysql_num_rows(sqline.res);
		
		for(i=0;i<sqline.fields;i++){
			sqline.row = mysql_fetch_row(sqline.res);
			
			if(stol(sqline.row[1])!=0)
				msend(thread,howtosend,snick,nick,"%d %s %s [%s]",i+1,sqline.row[0],sqline.row[3],get_localtime((time_t)stol(sqline.row[1])));
			else
				msend(thread,howtosend,snick,nick,"%d %s %s",i+1,sqline.row[0],sqline.row[3]);
		}
		send_lang(thread,language,howtosend,snick,nick,"END_OF_STATS_REPORT");

		mysql_free_result_myr(sqline.res); //free MySQL result
		
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O SQLINE Clear function for the O SQLINE Command.
 * 
 * The O SQLINE Clear Command allows an admin to<br/>
 *   delete all Sqlines.
 *  
 * Syntax is: /msg O SQLINE CLEAR
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL O_sqline_clear(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */

	/* Initialise Variables to (NULL) */

	/*
	 * Sqline Clear Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"sqline clear")){
		domysql(thread,"TRUNCATE `orion_os_sqlines`");
		send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
		
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the O SQLINE Command (/msg O HELP SQLINE)
 * 
 * When the user tries to get help for this command, using /msg O HELP SQLINE,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP SQLINE
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_sqline_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Sqline Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help sqline")) {
		if(!param[5]){
			send_lang(thread,language,howtosend,snick,nick,"O_HELP_SQLINE");
			setWebSuccess(thread); /* Indentify webform that changes were applied */
		}
			
		else send_lang(thread,language,howtosend,snick,nick,"O_NO_HELP_AVAIL");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief O Help List Command - SQLINE part (/msg O help)
 * 
 * When the user tries to get general O help, using /msg O help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the SQLINE command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_sqline_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_O_Level=(thread->sql.aliases.active>0 ? thread->sql.access.olevel : is_user);
	/*
	 * Sqline (Help listing)
	 */
	if(Users_O_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"O_SHOWCOMMANDS_SQLINE",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"sqline"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
