/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file o_trustdeny.c
 *   @brief This is the Module file for O TRUSTDENY Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the O TRUSTDENY command.
 *
 *   The O TRUSTDENY Command allows an admin to manipulate the Session-Forbid-List.<br/>
 *   Hosts / Ips in this list cannot be the target of a TRUST modifiction. (Trust forbid)
 *
 *   Syntax is: /msg O TRUSTDENY { ADD | DEL | LIST } ...
 *
 *<table border="0">
 *    <tr><td> ADD	</td><td> Syntax is: /msg O TRUSTDENY ADD	</td><td> { < host > | < ip > } < description >	</td><td> (Adds a deny for target host / ip)			</td></tr>
 *    <tr><td> DEL	</td><td> Syntax is: /msg O TRUSTDENY DEL	</td><td> { < host > | < ip > }					</td><td> (Deletes a deny for target host / ip)			</td></tr>
 *    <tr><td> LIST	</td><td> Syntax is: /msg O TRUSTDENY LIST	</td><td> [ *host-pattern* ]					</td><td> (Shows all deny's matching the given pattern)	</td></tr>
 *</table>
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"

#define SERVICE "O"
#define MODUL "trustdeny"
#define MODUL_FILE "o_trustdeny"
#define MODUL_VERSION "1.2"

MOD_BOOL O_trustdeny_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_trustdeny_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_trustdeny_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL O_trustdeny_add_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_trustdeny_del_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_trustdeny_list_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL O_trustdeny_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_trustdeny_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"trustdeny option[1]","trustdeny helpoption[1]");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,WEBACCESS,LOGINONLY,OPER_IF_DEFCON,OPERONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addLeastModuleLevels(thread,&Module,"trustdeny",850,INFINIT); /* just to add a displaylevel for /msg o help
															  this should always be the lowest used Module level! */

	addCommand(thread,&Module,"trustdeny add",O_trustdeny_add,850,INFINIT,5,0);
	addCommand(thread,&Module,"trustdeny del",O_trustdeny_del,850,INFINIT,5,5);
	addCommand(thread,&Module,"trustdeny list",O_trustdeny_list,850,INFINIT,4,5);

	addCommand(thread,&Module,"help trustdeny",O_trustdeny_help,850,INFINIT,4,4);
	addCommand(thread,&Module,"help trustdeny add",O_trustdeny_add_help,850,INFINIT,5,5);
	addCommand(thread,&Module,"help trustdeny del",O_trustdeny_del_help,850,INFINIT,5,5);
	addCommand(thread,&Module,"help trustdeny list",O_trustdeny_list_help,850,INFINIT,5,5);
	
	addCommand(thread,&Module,"help",O_trustdeny_help_listing,850,INFINIT,3,3); //100 - each Admin is able to see it.
	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief O TRUSTDENY Add function for the O TRUSTDENY Command.
 * 
 * The O TRUSTDENY Add Command allows an admin to Add a trustdeny for given host.<br/>
 *   A Trustdeny is like a Forbid from trusting it. (So it can't be added to the Trustlist again)
 *  
 * Syntax is: /msg O TRUSTDENY ADD { < host > | < ip > } < description >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_trustdeny_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	char nowhost[250];
	int index = 0;
	int i = 0;
	int trustconnections = 0;
	char *ip_tok[10];
	deepSQL killing;

	/* Initialise Variables to (NULL) */
	bzero(&nowhost,sizeof(nowhost));
	bzero(&ip_tok,sizeof(ip_tok));
	bzero(&killing,sizeof(killing));

	/*
	 * Trustdeny Add Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"trustdeny add")){
		dynncpy(nowhost,param[5],sizeof(nowhost));
		index = strsplit(nowhost,ip_tok,'.');
		if(ip_tok[1]){
			if(stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_os_trusts` WHERE `art`='0' AND `rhost`='%s'",mysql_quote(thread,param[5])))==0){
				add_Trust(param[5],_dstrjoin(' ',param,6),0,(Trust_art)0,1);
				domysql(thread,"INSERT `orion_os_trusts` (`art`,`rhost`,`description`) VALUES ('0','%s','%s')",mysql_quote(thread,param[5]),mysql_quote(thread,_dstrjoin(' ',param,6)));
				//domysql(thread,"UPDATE `orion_os_trusts` SET `art`='0',`description`='%s' WHERE `rhost`='%s'",mysql_quote(thread,_dstrjoin(' ',param,5)),mysql_quote(thread,param[4]));
					/***************************************TRUST EXTENSION (ADVANCED)*************************************/
					trustconnections = stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_core_condata` WHERE `rhost`='%s'",mysql_quote(thread,param[5])));
					killing.res      = doSelectMySQL(thread,"SELECT `nick` FROM `orion_core_condata` WHERE `rhost`='%s' ORDER BY `time` DESC LIMIT 0,%d",mysql_quote(thread,param[5]),abs(trustconnections-m->maxconnections));
					killing.fields   = mysql_num_rows(killing.res);

					for(i=0;i<killing.fields;i++){
						killing.row = mysql_fetch_row(killing.res);
						
						do_kill(thread,m->servername,killing.row[0],"Session limit exceeded"); //# die letzten killen
					}
					/* mysql_free_result_myr(); - killing */
					mysql_free_result_myr(killing.res);					
					/******************************************************************************************************/
				if(i!=0) send_lang(thread,language,howtosend,snick,nick,"O_TRUSTDENY_ADD_X_ITEMS_DONE",i+1);
				send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
				
				setWebSuccess(thread); /* Indentify webform that changes were applied */
			}else send_lang(thread,language,howtosend,snick,nick,"O_TRUSTDENY_ADD_TRUST_ALREADY_DENIED");
		}else send_lang(thread,language,howtosend,snick,nick,"INVALID_INPUT_NO_IP");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O TRUSTDENY Del function for the O TRUSTDENY Command.
 * 
 * The O TRUSTDENY Del Command allows an admin to delete the trustdeny for a given host.<br/>
 *   A Trustdeny is like a Forbid from trusting it. (So it can't be added to the Trustlist again)<br/>
 *   By deleting a Trustdeny it's open for adding it to the trustlist again
 *  
 * Syntax is: /msg O TRUSTDENY DEL { < host > | < ip > }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_trustdeny_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Trustdeny Del Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"trustdeny del")){
		if(stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_os_trusts` WHERE `rhost`='%s' AND `art`='0'",mysql_quote(thread,param[5])))){
			del_Trust(getTrust(param[5],(Trust_art)0));
			domysql(thread,"DELETE FROM `orion_os_trusts` WHERE `rhost`='%s' AND `art`='0'",mysql_quote(thread,param[5]));
			send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
			
			setWebSuccess(thread); /* Indentify webform that changes were applied */
		}else send_lang(thread,language,howtosend,snick,nick,"O_TRUSTDENY_DEL_NO_TRUSTDENY_FOR_HOST",param[5]);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O TRUSTDENY List function for the O TRUSTDENY Command.
 * 
 * The O TRUSTDENY List Command allows an admin to List trustdenys matching a given host.<br/>
 *   A Trustdeny is like a Forbid from trusting it. (So it can't be added to the Trustlist again)
 *  
 * Syntax is: /msg O TRUSTDENY LIST [ *hostpattern* ]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_trustdeny_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL trust;
	char pattern[BUFSIZE];
	int counter=0;

	/* Initialise Variables to (NULL) */
	bzero(&trust,sizeof(trust));
	bzero(&pattern,sizeof(pattern));

	/*
	 * Trustdeny List Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"trustdeny list")){
		if(param[5] && strcor(param[5],"*")){
			dyncpy (pattern,param[5]);
			strrepc(pattern,'*','%');
			trust.res = doSelectMySQL(thread,"SELECT `rhost`,`description` FROM `orion_os_trusts` WHERE `art`='0' AND `rhost` LIKE '%s' ORDER BY `rhost` ASC,`art` DESC",mysql_quote(thread,pattern));
		}
		else
			trust.res = doSelectMySQL(thread,"SELECT `rhost`,`description` FROM `orion_os_trusts` WHERE `art`='0'");

		while((trust.row = mysql_fetch_row(trust.res))){ //fetch res to row's
			send_lang(thread,language,howtosend,snick,nick,"O_TRUSTDENY_LIST_LIST",counter+1,trust.row[0],trust.row[1]);
			counter++;
		}
		mysql_free_result_myr(trust.res); //free MySQL result
		send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
		
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the O TRUSTDENY Command (/msg O HELP TRUSTDENY)
 * 
 * When the user tries to get help for this command, using /msg O HELP TRUSTDENY,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP TRUSTDENY
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_trustdeny_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Trustdeny Help
	 */ 
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help trustdeny")) {
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_TRUSTDENY");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O TRUSTDENY Add Help function for the O TRUSTDENY Command (/msg O HELP TRUSTDENY ADD)
 * 
 * When the user tries to get help for this command, using /msg O HELP TRUSTDENY ADD,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP TRUSTDENY ADD
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_trustdeny_add_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Trustdeny Add Help
	 */ 
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help trustdeny add")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"trustdeny add"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_TRUSTDENY_ADD");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O TRUSTDENY Del Help function for the O TRUSTDENY Command (/msg O HELP TRUSTDENY DEL)
 * 
 * When the user tries to get help for this command, using /msg O HELP TRUSTDENY DEL,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP TRUSTDENY DEL
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_trustdeny_del_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Trustdeny Del Help
	 */ 
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help trustdeny del")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"trustdeny del"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_TRUSTDENY_DEL");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O TRUSTDENY List Help function for the O TRUSTDENY Command (/msg O HELP TRUSTDENY LIST)
 * 
 * When the user tries to get help for this command, using /msg O HELP TRUSTDENY LIST,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP TRUSTDENY LIST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_trustdeny_list_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Trustdeny List Help
	 */ 
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help trustdeny list")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"trustdeny list"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_TRUSTDENY_LIST");
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief O Help List Command - TRUSTDENYADD part (/msg O help)
 * 
 * When the user tries to get general O help, using /msg O help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the TRUSTDENYADD command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_trustdeny_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_O_Level=(thread->sql.aliases.active>0 ? thread->sql.access.olevel : is_user);
	/*
	 * Trustdeny (Help listing)
	 */
	if(Users_O_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"O_SHOWCOMMANDS_TRUSTDENY",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"trustdeny"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
