/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file o_trustgroup.c
 *   @brief This is the Module file for O TRUSTGROUP Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the O TRUSTGROUP command.
 *
 *   The O TRUSTGROUP Command allows an admin to manipulate the Sessiongroups<br/>
 *   for specific ips / hosts. (Manages the limits/names for the Sessiongroups)
 *
 *   Syntax is: /msg O TRUSTGROUP { ADD | DEL | MODIFY | EDIT | LIST } ...
 *
 *<table border="0">
 *    <tr><td> ADD		</td><td> Syntax is: /msg O TRUSTGROUP ADD		</td><td> < #trustgroup > < limit >				</td><td> (Adds a trustgroup with a virtual limit)	</td></tr>
 *    <tr><td> DEL		</td><td> Syntax is: /msg O TRUSTGROUP DEL		</td><td> < #trustgroup >						</td><td> (Deletes an existing trustgroup)			</td></tr>
 *    <tr><td> MODIFY	</td><td> Syntax is: /msg O TRUSTGROUP MODIFY	</td><td> < #trustgroup > < limit >				</td><td> (Changes the trustgroup's limit)			</td></tr>
 *    <tr><td> EDIT		</td><td> Syntax is: /msg O TRUSTGROUP EDIT		</td><td> < #trustgroup > < #New-trustgroup >	</td><td> (Renames the trustgroup)					</td></tr>
 *    <tr><td> LIST		</td><td> Syntax is: /msg O TRUSTGROUP LIST		</td><td>										</td><td> (Shows all existing trustgroups)			</td></tr>
 *</table>
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"

#define SERVICE "O"
#define MODUL "trustgroup"
#define MODUL_FILE "o_trustgroup"
#define MODUL_VERSION "1.2"

MOD_BOOL O_trustgroup_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_trustgroup_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_trustgroup_modify(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_trustgroup_edit(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_trustgroup_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL O_trustgroup_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_trustgroup_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"trustgroup option[1]","trustgroup");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,WEBACCESS,LOGINONLY,OPER_IF_DEFCON,OPERONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addLeastModuleLevels(thread,&Module,"trustgroup",840,INFINIT); /* just to add a displaylevel for /msg o help
															  this should always be the lowest used Module level! */

	addCommand(thread,&Module,"trustgroup add",O_trustgroup_add,840,INFINIT,6,6);
	addCommand(thread,&Module,"trustgroup del",O_trustgroup_del,840,INFINIT,5,5);
	addCommand(thread,&Module,"trustgroup modify",O_trustgroup_modify,840,INFINIT,6,6);
	addCommand(thread,&Module,"trustgroup edit",O_trustgroup_edit,840,INFINIT,6,6);
	addCommand(thread,&Module,"trustgroup list",O_trustgroup_list,840,INFINIT,4,4);
	
	addCommand(thread,&Module,"help trustgroup",O_trustgroup_help,840,INFINIT,4,4);
	addCommand(thread,&Module,"help",O_trustgroup_help_listing,840,INFINIT,3,3); //100 - each Admin is able to see it.

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief O TRUSTGROUP Add function for the O TRUSTGROUP Command.
 * 
 * The O TRUSTGROUP Add Command allows the user to add a trustgroup with<br/>
 *    a specific GroupLimit. (Each Trust in that group has the same SessionLimit)
 *  
 * Syntax is: /msg O TRUSTGROUP ADD < #trustgroup > < limit >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_trustgroup_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL trustgroup;
	char group[60];

	/* Initialise Variables to (NULL) */
	bzero(&trustgroup,sizeof(trustgroup));
	bzero(&group,sizeof(group));

	/*
	 * Trustgroup Add Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"trustgroup add")){
		if(streqnc(param[5],"#main")) send_lang(thread,language,howtosend,snick,nick,"O_TRUSTGROUP_PERMISSION_DENIED_TO_MAIN_GROUP");
		else if(CheckOption(thread,snick,nick,READONLY)){
			if(digitcol(param[6])){
				dynncpy(group,param[5],sizeof(group));
				
				if(group[0]!='#')
					strunshift("#",group,sizeof(group));
							
				trustgroup.res    = doSelectMySQL(thread,"SELECT `group`,`limit` FROM `orion_os_trustgroups` WHERE `group`='%s'",mysql_quote(thread,group));
				trustgroup.fields = mysql_num_rows(trustgroup.res);
				trustgroup.row    = mysql_fetch_row(trustgroup.res);	

				if(trustgroup.fields)
					send_lang(thread,language,howtosend,snick,nick,"O_TRUSTGROUP_GROUP_ALREADY_EXISTS",stol(trustgroup.row[1]));
						
				else{
					add_Trustgroup(group,stol(param[6]),1);
					domysql(thread,"INSERT `orion_os_trustgroups` (`group`,`limit`) VALUES ('%s','%ld')",mysql_quote(thread,group),stol(param[6]));
					send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
					
					setWebSuccess(thread); /* Indentify webform that changes were applied */
				}

				mysql_free_result_myr(trustgroup.res); //free MySQL result
			}else send_lang(thread,language,howtosend,snick,nick,"O_TRUSTGROUP_INVALID_OPTION");
		}else send_lang(thread,language,howtosend,snick,nick,"READONLY_MODE");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O TRUSTGROUP Del function for the O TRUSTGROUP Command.
 * 
 * The O TRUSTGROUP Del Command allows an admin to delete a trustgroup from<br/>
 *    the list (All trusts in that group will be deleted too!).
 *  
 * Syntax is: /msg O TRUSTGROUP DEL < #trustgroup >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_trustgroup_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL group;
	deepSQL killing;
	char tgroup[60];
	
	int trustconnections = 0;
	int i = 0, j = 0;

	/* Initialise Variables to (NULL) */
	bzero(&group,sizeof(group));
	bzero(&killing,sizeof(killing));
	bzero(&tgroup,sizeof(tgroup));

	/*
	 * Trustgroup Del Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"trustgroup del")){
		if(streqnc(param[5],"#main")) send_lang(thread,language,howtosend,snick,nick,"O_TRUSTGROUP_PERMISSION_DENIED_TO_MAIN_GROUP");
		else if(CheckOption(thread,snick,nick,READONLY)){
			dynncpy(tgroup,param[5],sizeof(tgroup));
			if(tgroup[0]!='#')
				strunshift("#",tgroup,sizeof(tgroup));

			if(stoi(mysql_sel_var(thread,"SELECT count(1) FROM `orion_os_trustgroups` WHERE `group`='%s'",mysql_quote(thread,tgroup)))==1){
				/***************************************TRUST EXTENSION (ADVANCED)*************************************/
				if(m->OSForceTrusts){
					group.res    = doSelectMySQL(thread,"SELECT `rhost` FROM `orion_os_trusts` WHERE `limit`='%s'",mysql_quote(thread,tgroup));
					group.fields = mysql_num_rows(group.res);
						
					for(j=0;j<group.fields;j++){
						group.row = mysql_fetch_row(group.res);
							
						trustconnections = stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_core_condata` WHERE `rhost`='%s'",mysql_quote(thread,group.row[0])));
						killing.res      = doSelectMySQL(thread,"SELECT `nick` FROM `orion_core_condata` WHERE `rhost`='%s' ORDER BY `time` DESC LIMIT 0,%d",mysql_quote(thread,group.row[0]),abs(trustconnections-m->maxconnections));
						killing.fields   = mysql_num_rows(killing.res);
							
						if(killing.fields<m->MaxSessionKill){
							for(i=0;i<killing.fields;i++){
								killing.row = mysql_fetch_row(killing.res);
								
								if(m->SessionLimitExceeded[0]!='\0')
									do_kill(thread,m->servername,killing.row[0],_dynadd(m->SessionLimitExceeded,group.row[0]));
								else
									do_kill(thread,m->servername,killing.row[0],"Session limit exceeded"); //# die letzten killen 
							}
						}
						else{
							char rhost[MAX_HOST_LEN];
							bzero(&rhost,sizeof(rhost));

							straddns(rhost,"*@",sizeof(rhost));
							straddns(rhost,group.row[0],sizeof(rhost));

							if(m->SessionLimitExceeded[0]!='\0')
								do_gline(thread,m->servername,rhost,_dynadd("%s!%s@%s",O->nick,O->ident,O->host),(m->SessionGlineExpiry+time(NULL)),time(NULL),_dynadd(m->SessionLimitExceeded,group.row[0]));
							else
								do_gline(thread,m->servername,rhost,_dynadd("%s!%s@%s",O->nick,O->ident,O->host),(m->SessionGlineExpiry+time(NULL)),time(NULL),"Session limit exceeded");
						}
						/* mysql_free_result_myr(); - killing */
						mysql_free_result_myr(killing.res);
					}
					/* mysql_free_result_myr(); - group */
					mysql_free_result_myr(group.res);
				}
				/******************************************************************************************************/
				del_Trustgroup(getTrustgroup(tgroup));
				del_Trust_by_Group(tgroup,(Trust_art)1);
				domysql(thread,"DELETE FROM `orion_os_trustgroups` WHERE `group`='%s'",mysql_quote(thread,tgroup));
				domysql(thread,"DELETE FROM `orion_os_trusts` WHERE `limit`='%s'",mysql_quote(thread,tgroup));
				send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
				
				setWebSuccess(thread); /* Indentify webform that changes were applied */
			}else send_lang(thread,language,howtosend,snick,nick,"O_TRUSTGROUP_GROUP_DOES_NOT_EXISTS");
		}else send_lang(thread,language,howtosend,snick,nick,"READONLY_MODE");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O TRUSTGROUP Modify function for the O TRUSTGROUP Command.
 * 
 * The O TRUSTGROUP Modify Command allows the user to modify a trustgroup which<br/>
 *    is used to change the Group's Limit. (Each Trust in that group has the same SessionLimit)
 *  
 * Syntax is: /msg O TRUSTGROUP MODIFY < #trustgroup > < New-limit >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_trustgroup_modify(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL trustgroup;
	deepSQL group;
	deepSQL killing;
	char tgroup[60];
	int trustconnections = 0;
	int i = 0, j = 0;

	/* Initialise Variables to (NULL) */
	bzero(&trustgroup,sizeof(trustgroup));
	bzero(&group,sizeof(group));
	bzero(&killing,sizeof(killing));
	bzero(&tgroup,sizeof(tgroup));

	/*
	 * Trustgroup Modify Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"trustgroup modify")){
		if(streqnc(param[5],"#main")) send_lang(thread,language,howtosend,snick,nick,"O_TRUSTGROUP_PERMISSION_DENIED_TO_MAIN_GROUP");
		else if(CheckOption(thread,snick,nick,READONLY)){
			if(digitcol(param[6])){
				//bzero(&trustgroup,sizeof(trustgroup));

				dynncpy(tgroup,param[5],sizeof(tgroup));
				if(tgroup[0]!='#')
					strunshift("#",tgroup,sizeof(tgroup));

				trustgroup.res    = doSelectMySQL(thread,"SELECT `group`,`limit` FROM `orion_os_trustgroups` WHERE `group`='%s'",mysql_quote(thread,tgroup));
				trustgroup.row    = mysql_fetch_row(trustgroup.res);
				trustgroup.fields = mysql_num_rows(trustgroup.res);

				if(trustgroup.fields){
					if(streqnc(trustgroup.row[0],tgroup)){
						getTrustgroup(tgroup)->limit=stol(param[6]);
						domysql(thread,"UPDATE `orion_os_trustgroups` SET `limit`='%ld' WHERE `group`='%s'",stol(param[6]),mysql_quote(thread,tgroup));
						/***************************************TRUST EXTENSION (ADVANCED)*************************************/
						if(m->OSForceTrusts){
							group.res    = doSelectMySQL(thread,"SELECT `rhost` FROM `orion_os_trusts` WHERE `limit`='%s'",mysql_quote(thread,tgroup));
							group.fields = mysql_num_rows(group.res);
								
							for(j=0;j<group.fields;j++){
								group.row = mysql_fetch_row(group.res);
								
								trustconnections = stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_core_condata` WHERE `rhost`='%s'",mysql_quote(thread,group.row[0])));
								killing.res      = doSelectMySQL(thread,"SELECT `nick` FROM `orion_core_condata` WHERE `rhost`='%s' ORDER BY `time` DESC LIMIT 0,%d",mysql_quote(thread,group.row[0]),abs(trustconnections-m->maxconnections));
								killing.fields   = mysql_num_rows(killing.res);
				
								if(killing.fields<m->MaxSessionKill){
									for(i=0;i<killing.fields;i++){
										killing.row = mysql_fetch_row(killing.res);
									
										if(m->SessionLimitExceeded[0]!='\0')
											do_kill(thread,m->servername,killing.row[0],_dynadd(m->SessionLimitExceeded,group.row[0]));
										else
											do_kill(thread,m->servername,killing.row[0],"Session limit exceeded"); //# die letzten killen  
									}
								}
								else{
									char rhost[MAX_HOST_LEN];
									bzero(&rhost,sizeof(rhost));

									straddns(rhost,"*@",sizeof(rhost));
									straddns(rhost,group.row[0],sizeof(rhost));

									if(m->SessionLimitExceeded[0]!='\0')
										do_gline(thread,m->servername,rhost,_dynadd("%s!%s@%s",O->nick,O->ident,O->host),(m->SessionGlineExpiry+time(NULL)),time(NULL),_dynadd(m->SessionLimitExceeded,group.row[0]));
									else
										do_gline(thread,m->servername,rhost,_dynadd("%s!%s@%s",O->nick,O->ident,O->host),(m->SessionGlineExpiry+time(NULL)),time(NULL),"Session limit exceeded");
								}
								/* mysql_free_result_myr(); - killing */
								mysql_free_result_myr(killing.res);
							}
							/* mysql_free_result_myr(); - group */
							mysql_free_result_myr(group.res);
						}
						/******************************************************************************************************/
						send_lang(thread,language,howtosend,snick,nick,"O_TRUSTGROUP_SET_NEW_VIRTUAL_LIMIT_TO_GROUP",stol(trustgroup.row[1]),tgroup);
						
						setWebSuccess(thread); /* Indentify webform that changes were applied */
					}else send_lang(thread,language,howtosend,snick,nick,"O_TRUSTGROUP_GROUP_DOES_NOT_EXISTS");
				}else send_lang(thread,language,howtosend,snick,nick,"O_TRUSTGROUP_GROUP_DOES_NOT_EXISTS");

				mysql_free_result_myr(trustgroup.res); //free MySQL result
			}else send_lang(thread,language,howtosend,snick,nick,"INVALID_PARAMETER",param[6]);
		}else send_lang(thread,language,howtosend,snick,nick,"READONLY_MODE");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O TRUSTGROUP Edit function for the O TRUSTGROUP Command.
 * 
 * The O TRUSTGROUP Edit Command allows an admin to edit a trustgroup which<br/>
 *    changes the Name of target TrustGroup. (Each Trust in that group has the same SessionLimit)
 *  
 * Syntax is: /msg O TRUSTGROUP EDIT < #trustgroup > < #New-trustgroup-Name >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_trustgroup_edit(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL trustgroup;
	char new_group[60];
	char old_group[60];

	/* Initialise Variables to (NULL) */
	bzero(&trustgroup,sizeof(trustgroup));
	bzero(&new_group,sizeof(new_group));
	bzero(&old_group,sizeof(old_group));
	
	/*
	 * Trustgroup Edit Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"trustgroup edit")){
		if(streqnc(param[5],"#main")) send_lang(thread,language,howtosend,snick,nick,"O_TRUSTGROUP_PERMISSION_DENIED_TO_MAIN_GROUP");
		else if(CheckOption(thread,snick,nick,READONLY)){
			
			dynncpy(new_group,param[6],sizeof(new_group));
			if(new_group[0]!='#')
				strunshift("#",new_group,sizeof(new_group));
						
			dynncpy(old_group,param[5],sizeof(old_group));
			if(old_group[0]!='#')
				strunshift("#",old_group,sizeof(old_group));
						
			bzero(&trustgroup,sizeof(trustgroup));
			trustgroup.res    = doSelectMySQL(thread,"SELECT `group`,`limit` FROM `orion_os_trustgroups` WHERE `group`='%s'",mysql_quote(thread,old_group));
			trustgroup.row    = mysql_fetch_row(trustgroup.res);
			trustgroup.fields = mysql_num_rows(trustgroup.res);

			if(trustgroup.fields && stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_os_trustgroups` WHERE `group`='%s'",mysql_quote(thread,new_group)))==0){
				dynncpy(getTrustgroup(old_group)->group,new_group,sizeof(getTrustgroup(old_group)->group));
				edit_Trust_by_Group(old_group,new_group,(Trust_art)1);

				domysql(thread,"UPDATE `orion_os_trustgroups` SET `group`='%s' WHERE `group`='%s'",mysql_quote(thread,new_group),mysql_quote(thread,old_group));
				domysql(thread,"UPDATE `orion_os_trusts` SET `limit`='%s' WHERE `limit`='%s' AND `art`='1'",mysql_quote(thread,new_group),mysql_quote(thread,old_group));
				send_lang(thread,language,howtosend,snick,nick,"O_TRUSTGROUP_GROUP_X_CHANGED_TO_GROUP_Y",old_group,new_group);
				
				setWebSuccess(thread); /* Indentify webform that changes were applied */
			}else send_lang(thread,language,howtosend,snick,nick,"O_TRUSTGROUP_GROUP_DOES_NOT_EXISTS");

			mysql_free_result_myr(trustgroup.res); //free MySQL result
		}else send_lang(thread,language,howtosend,snick,nick,"READONLY_MODE");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O TRUSTGROUP List function for the O TRUSTGROUP Command.
 * 
 * The O TRUSTGROUP List Command allows an admin to list all trustgroups
 *  
 * Syntax is: /msg O TRUSTGROUP LIST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_trustgroup_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL maingroup;
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&maingroup,sizeof(maingroup));

	/*
	 * Trustgroup List Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"trustgroup list")){
		maingroup.res    = doSelectMySQL(thread,"SELECT `group`,`limit` FROM `orion_os_trustgroups`");
		maingroup.fields = mysql_num_rows(maingroup.res);

		for(i=0;i<maingroup.fields;i++){
			maingroup.row = mysql_fetch_row(maingroup.res);
						
			send_lang(thread,language,howtosend,snick,nick,"O_TRUSTGROUP_LIST_GROUPS",maingroup.row[0],stol(maingroup.row[1]));
		}
		mysql_free_result_myr(maingroup.res); //free MySQL result

		send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
		
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the O TRUSTGROUP Command (/msg O HELP TRUSTGROUP)
 * 
 * When the user tries to get help for this command, using /msg O HELP TRUSTGROUP,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP TRUSTGROUP
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_trustgroup_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Trustgroup Help
	 */ 
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help trustgroup")){
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"trustgroup"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_TRUSTGROUP",snick,snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"O_NO_HELP_AVAIL");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief O Help List Command - TRUSTGROUP part (/msg O help)
 * 
 * When the user tries to get general O help, using /msg O help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the TRUSTGROUP command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_trustgroup_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_O_Level=(thread->sql.aliases.active>0 ? thread->sql.access.olevel : is_user);
	/*
	 * Trustgroup (Help listing)
	 */
	if(Users_O_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"O_SHOWCOMMANDS_TRUSTGROUP",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"trustgroup"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
