/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file q_chanflags.c
 *   @brief This is the Module file for Q CHANFLAGS Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the Q CHANFLAGS command.
 *
 *   The Q CHANFLAGS Command allows the user to change/view<br/>
 *   target channel's Channel-Flags.
 *
 *   Available Chanflags are:<br/>
 *    +b - Bitch<br/>
 *         Prevents anyone who hasn't got operator privs on the channel from getting OP.<br/>
 *    +c - Channel-limit<br/>
 *         Activate channel-limit protection. - Set parameters with AUTOLIMIT command.<br/>
 *    +f - Force topic<br/>
 *         Prevent people change the topic with /topic command - SETTOPIC is needed<br/>
 *    +k - Force key<br/>
 *         Set a key permanent on the channel. - Set the key with the KEY command.<br/>
 *    +l - Force limit<br/>
 *         Set a limit permanent on the channel. - Set the limit with the LIMIT command.<br/>
 *    +p - Protect opers<br/>
 *         Prevent people who have with OP privs on the channel from being deopped.<br/>
 *    +t - Topic save<br/>
 *         Allow the bot to set the topic if it was restarted.<br/>
 *    +w - Welcome message<br/>
 *         Send a welcome message (set by WELCOME) to all joining users.<br/>
 *    +o - AutoOp All<br/>
 *         AutoOps all joining users. (useful for private channels)<br/>
 *    +v - AutoVoice All<br/>
 *         AutoVoices all joining users. (useful for private channels)
 *
 *   Syntax is: /msg Q CHANFLAGS #channel [ [+|-]Flags ]
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"

#define SERVICE "Q"
#define MODUL "chanflags"
#define MODUL_FILE "q_chanflags"
#define MODUL_VERSION "1.2"

char Q_Avail_Chanmodes[] = "bcfkloptvw";

MOD_BOOL Q_chanflags(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_chanflags_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_chanflags_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"chanflags","chanflags");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,WEBACCESS,LOGINONLY,OPER_IF_DEFCON);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"chanflags",Q_chanflags,550,1,4,5);
	addCommand(thread,&Module,"help chanflags",Q_chanflags_help,100,1,4,4);
	addCommand(thread,&Module,"showcommands",Q_chanflags_showcommands,100,1,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the Q CHANFLAGS Command.
 * 
 * The Q CHANFLAGS Command allows the user to change/view<br/>
 *   target channel's Channel-Flags.
 *
 * Syntax is: /msg Q CHANFLAGS #channel [ [+|-]Flags ]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_chanflags(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global variables */
	char chanadd[MAX_MODES]; //addmodes[SIZE] - SIZE, max. chanflags per channel (Q chanflags, Q chanmodes not IRCD chanmodes), modes for addition sequence
	char chandel[MAX_MODES]; //delmodes[SIZE] - SIZE, max. chanflags per channel (Q chanflags, Q chanmodes not IRCD chanmodes), modes for delete sequence
	char type;
	char chr;
	size_t i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&chanadd,sizeof(chanadd));
	bzero(&chandel,sizeof(chandel));
	chr='\0';
	type='\0';

	/*
	 * Chanflags Command
	 */
	if(thread->sql.access.qlevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"chanflags")){
		/* Updates the Threads Channel-Management */
		UpdateChannelInformation(thread,param[4],NULL);

		if(thread->c.chan.fields && streqnc(thread->c.chan.serv,"q")){
			if((thread->c.chan.settings & CS_FORBID_BIT))		send_lang(thread,language,howtosend,snick,nick,"CHANNEL_FORBIDDEN",thread->c.chan.serv,thread->c.chan.reason);
			else if((thread->c.chan.settings & CS_SUSPEND_BIT))	send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
			else if(param[4] && !param[5]){
				if(thread->sql.access.qlevel>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"chanflags") || strcor(thread->c.chan.access.ownlevel,"hmnoptv")){
					if(thread->c.chan.flags[0]!='\0'){
						send_lang(thread,language,howtosend,snick,nick,"Q_CHANFLAGS_HEADER");
						send_lang(thread,language,howtosend,snick,nick,"Q_CHANFLAGS_CURRENT_CHANFLAGS",param[4],thread->c.chan.flags);
						if(strcor(thread->c.chan.flags,"c"))
							send_lang(thread,language,howtosend,snick,nick,"Q_CHANFLAGS_AUTOLIMIT",thread->c.chan.alimit);
						if(strcor(thread->c.chan.flags,"l"))
							send_lang(thread,language,howtosend,snick,nick,"Q_CHANFLAGS_LIMIT",thread->c.chan.limit);
							
						setWebSuccess(thread); /* Indentify webform that changes were applied */
					}else send_lang(thread,language,howtosend,snick,nick,"Q_NO_CHANFLAGS_SET");
				}else send_lang(thread,language,howtosend,snick,nick,"NOT_KNOWN_ON_CHANNEL",param[4]);
			}else if(strcor(param[5],Q_Avail_Chanmodes) && CheckOption(thread,snick,nick,READONLY)){ //const char Avail_Chanmodes[20]="bcfkloptvw";
				if(thread->sql.access.qlevel>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"chanflags") || strcor(thread->c.chan.access.ownlevel,"nm")){
					//type='\0';
					//chanadd[0]='\0';
					//chandel[0]='\0';
					for(i=0;i<dynlen(param[5]);i++){
						chr=param[5][i];
						if(chr=='+' || chr=='-') type=param[5][i];
						else if(strcin(chr,Q_Avail_Chanmodes)){ //const char Q_Avail_Chanmodes[20]="bcfkloptvw";
							if(type=='+' && !strcin(chr,chanadd) && !strcin(chr,thread->c.chan.flags)){
								straddnc(chanadd,chr,sizeof(chanadd));
								//a++;
							}else if(type=='-' && !strcin(chr,chandel)){
								straddnc(chandel,chr,sizeof(chandel));
								//b++;
							}
						}
					}
					
					if(chanadd[0]!='\0'){
                        straddns(thread->c.chan.flags,chanadd,sizeof(thread->c.chan.flags));
						if(chandel[0]!='\0'){
							for(i=0;i<strlen(chandel);i++)
								strdelc(thread->c.chan.flags,chandel[i]);
						}
						sort(thread->c.chan.flags);
							
						domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`flags`='%s' WHERE `channel`='%s' AND `serv`='Q'",time(NULL),mysql_quote(thread,thread->c.chan.flags),mysql_quote(thread,param[4]));
					}
					else if(chandel[0]!='\0'){
						for(i=0;i<strlen(chandel);i++) strdelc(thread->c.chan.flags,chandel[i]);
						sort(thread->c.chan.flags);
						domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`flags`='%s' WHERE `channel`='%s' AND `serv`='Q'",time(NULL),mysql_quote(thread,thread->c.chan.flags),mysql_quote(thread,param[4]));
					}
					
					if(chanadd[0]!='\0' || chandel[0]!='\0'){
						send_lang(thread,language,howtosend,snick,nick,"Q_COMMAND_DONE");
						setWebSuccess(thread); /* Indentify webform that changes were applied */
					}
					else
						send_lang(thread,language,howtosend,snick,nick,"Q_INVALID_CHANFLAGS");
				}else send_lang(thread,language,howtosend,snick,nick,"Q_RIGHT_MASTER_TO_CHANGE_FLAGS");
			}else send_lang(thread,language,howtosend,snick,nick,"Q_INVALID_CHANFLAGS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_EXISTING");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the Q CHANFLAGS Command (/msg Q HELP CHANFLAGS)
 * 
 * When the user tries to get help for this command, using /msg Q HELP CHANFLAGS,<br/>
 * this function is executed.
 * 
 * Syntax: /msg Q HELP CHANFLAGS
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_chanflags_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Chanflags Help
	 */
	if(thread->sql.access.qlevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help chanflags")){
		send_lang(thread,language,howtosend,snick,nick,"Q_HELP_CHANFLAGS",snick,snick,snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}

	else send_lang(thread,language,howtosend,snick,nick,"Q_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Q SHOWCOMMANDS Command - CHANFLAGS part (/msg Q showcommands)
 * 
 * When the user tries to get general Q help, using /msg Q showcommands, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to<br/>
 * include the information for the CHANFLAGS command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_chanflags_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=0, Users_Q_Level=0;
	command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"showcommands");
	Users_Q_Level=(thread->sql.aliases.active>0 ? thread->sql.access.qlevel : is_user);
	/*
	 * Chanflags (Showcommands)
	 */
	if(Users_Q_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"Q_SHOWCOMMANDS_CHANFLAGS",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"chanflags"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
