/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file q_suspend.c
 *   @brief This is the Module file for Q SUSPEND Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the Q SUSPEND command.
 *
 *   The Q SUSPEND Command allows an admin to disallow anyone from<br/>
 *   registering or using the given #channel.
 *
 *   May be cancelled by using the UNSUSPEND command to preserve all<br/>
 *   previous channel data/settings.<br/>
 *   (Reason may be required on certain networks.)<br/>
 *     Q suspends a channel and kick bans all users (admins only).
 *
 *   Syntax is: /msg Q SUSPEND #channel [ < reason > ]
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"

#define SERVICE "Q"
#define MODUL "suspend"
#define MODUL_FILE "q_suspend"
#define MODUL_VERSION "1.2"

MOD_BOOL Q_suspend(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_suspend_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_suspend_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"suspend","suspend");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,5,WEBACCESS,LOGINONLY,OPER_IF_DEFCON,OPERONLY,READONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"suspend",Q_suspend,990,INFINIT,5,0);
	addCommand(thread,&Module,"help suspend",Q_suspend_help,990,INFINIT,4,4);
	addCommand(thread,&Module,"showcommands",Q_suspend_showcommands,990,INFINIT,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the Q SUSPEND Command.
 * 
 * The Q SUSPEND Command allows an admin to disallow anyone from<br/>
 *   registering or using the given #channel.
 *
 *   May be cancelled by using the UNSUSPEND command to preserve all<br/>
 *   previous channel data/settings.<br/>
 *   (Reason may be required on certain networks.)<br/>
 *     Q suspends a channel and kick bans all users (admins only).
 *
 * Syntax is: /msg Q SUSPEND #channel [ < reason > ]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_suspend(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global variables */
	deepSQL nicks;
	char CBot[MAX_NICK_LEN];
	char *reason = NULL;
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&nicks,sizeof(nicks));
	bzero(&CBot,sizeof(CBot));

	/*
	 * Suspend Command
	 */
	if(thread->sql.access.qlevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"suspend")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		/* Set Q to our Bot, if BotServ is not loaded, or no Bot is assigned, or if SYMBIOSIS is OFF */
		QCheckBot(CBot,thread->c.chan.bot,thread->c.chan.botserv.set,sizeof(CBot));
		
		if(thread->c.chan.fields && streqnc(thread->c.chan.serv,"q")){
			if(thread->c.chan.settings & CS_FORBID_BIT)			send_lang(thread,language,howtosend,snick,nick,"CHANNEL_FORBIDDEN",thread->c.chan.serv,thread->c.chan.reason);
			else if(thread->c.chan.settings & CS_SUSPEND_BIT)	send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
			else{
				reason = _dstrjoin(' ',param,5);
				
				/* Update memory for caching */
#ifndef USE_THREADS
				thread->c.chan.expire = 0;
				thread->c.chan.settings |= CS_SUSPEND_BIT;
				//bzero(&thread->c.chan.reason,sizeof(thread->c.chan.reason));
				dynncpy(thread->c.chan.reason,reason,sizeof(thread->c.chan.reason));
#else
				thread->c.chan.settings |= CS_SUSPEND_BIT;
#endif

				/* Update SQL as live update */
				domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld', `expire`='0', `settings`='%ld', `reason`='%s' WHERE `channel`='%s' AND `serv`='Q'",time(NULL),thread->c.chan.settings,mysql_quote(thread,reason),mysql_quote(thread,param[4]));

				nicks.res    = doSelectMySQL(thread,"SELECT `orion_chan_users`.`nick`,`orion_core_condata`.`vhost` FROM `orion_chan_users` LEFT JOIN `orion_core_condata` ON `orion_chan_users`.`nick`=`orion_core_condata`.`nick` WHERE `channel`='%s' AND `orion_core_condata`.`faketype`='0'",mysql_quote(thread,param[4]));
				nicks.fields = mysql_num_rows(nicks.res);
		
				do_mode(thread,CBot,param[4],thread->c.chan.datecreate,IRC.OnSuspend);
				
				for(i=0;i<nicks.fields;i++){
					nicks.row = mysql_fetch_row(nicks.res);
			
					do_mode(thread,CBot,param[4],thread->c.chan.datecreate,"+b %s",nicks.row[1]);
					do_kick(thread,CBot,param[4],nicks.row[0],"channel is suspended reason: %s",reason);
				}
				/* mysql_free_result_myr(); - Chanaccess */
				mysql_free_result_myr(nicks.res);

				send_lang(thread,language,howtosend,snick,nick,"Q_COMMAND_DONE");
				setWebSuccess(thread); /* Indentify webform that changes were applied */
			}
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_EXISTING");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the Q SUSPEND Command (/msg Q HELP SUSPEND)
 * 
 * When the user tries to get help for this command, using /msg Q HELP SUSPEND,<br/>
 * this function is executed.
 * 
 * Syntax: /msg Q HELP SUSPEND
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_suspend_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Suspend Help
	 */
	if(thread->sql.access.qlevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help suspend")){
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"suspend"));
		send_lang(thread,language,howtosend,snick,nick,"Q_HELP_SUSPEND",snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"Q_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Q SHOWCOMMANDS Command - SUSPEND part (/msg Q showcommands)
 * 
 * When the user tries to get general Q help, using /msg Q showcommands, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to<br/>
 * include the information for the SUSPEND command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_suspend_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"showcommands");
	int Users_Q_Level=(thread->sql.aliases.active>0 ? thread->sql.access.qlevel : is_user);
	/*
	 * Suspend (Showcommands)
	 */
	if(Users_Q_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"Q_SHOWCOMMANDS_SUSPEND",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"suspend"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
