/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file q_unsuspend.c
 *   @brief This is the Module file for Q UNSUSPEND Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the Q UNSUSPEND command.
 *
 *   The Q UNSUSPEND Command allows an admin to release a suspended channel.<br/>
 *   All data and settings are preserved from before the suspension. (admins only).
 *
 *   Syntax is: /msg Q UNSUSPEND #channel
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"
#include "webform.h"

#define SERVICE "Q"
#define MODUL "unsuspend"
#define MODUL_FILE "q_unsuspend"
#define MODUL_VERSION "1.2"

MOD_BOOL Q_unsuspend(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_unsuspend_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_unsuspend_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"unsuspend","unsuspend");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s [$Rev: 365 $]",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,5,WEBACCESS,LOGINONLY,OPER_IF_DEFCON,OPERONLY,READONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"unsuspend",Q_unsuspend,990,INFINIT,4,4);
	addCommand(thread,&Module,"help unsuspend",Q_unsuspend_help,990,INFINIT,4,4);
	addCommand(thread,&Module,"showcommands",Q_unsuspend_showcommands,990,INFINIT,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the Q UNSUSPEND Command.
 * 
 * The Q UNSUSPEND Command allows an admin to release a suspended channel.<br/>
 *   All data and settings are preserved from before the suspension. (admins only).
 *
 * Syntax is: /msg Q UNSUSPEND #channel
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_unsuspend(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global variables */
	char CBot[MAX_NICK_LEN];
	
	/* Initialise Variables to (NULL) */
	bzero(&CBot,sizeof(CBot));
	
	/*
	 * Unsuspend Command
	 */
	if(thread->sql.access.qlevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"unsuspend")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		/* Set Q to our Bot, if BotServ is not loaded, or no Bot is assigned, or if SYMBIOSIS is OFF */
		QCheckBot(CBot,thread->c.chan.bot,thread->c.chan.botserv.set,sizeof(CBot));
		
		if(!thread->c.chan.fields)
			send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
		else if(thread->c.chan.settings & CS_SUSPEND_BIT){
			if(streqnc(thread->c.chan.serv,"q")){
				
				/* Update memory for caching */
#ifndef USE_THREADS
				thread->c.chan.expire = 1;
				thread->c.chan.settings &= ~CS_SUSPEND_BIT;
				bzero(&thread->c.chan.reason,sizeof(thread->c.chan.reason));
#else
				thread->c.chan.settings &= ~CS_SUSPEND_BIT;
#endif
				
				/* Update SQL as live update */
				domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`expire`='1',`reason`='',`settings`='%d' WHERE `channel`='%s' AND `serv`='Q'",time(NULL),thread->c.chan.settings,mysql_quote(thread,param[4]));
				unbanall(thread,CBot,param[4]);
	
				do_mode(thread,CBot,param[4],thread->c.chan.datecreate,IRC.OnUnSuspend);
				send_lang(thread,language,howtosend,snick,nick,"Q_COMMAND_DONE");
				
				setWebSuccess(thread); /* Indentify webform that changes were applied */
			}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_ALREADY_REGISTERED",thread->c.chan.serv);
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_SUSPENDED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the Q UNSUSPEND Command (/msg Q HELP UNSUSPEND)
 * 
 * When the user tries to get help for this command, using /msg Q HELP UNSUSPEND,<br/>
 * this function is executed.
 * 
 * Syntax: /msg Q HELP UNSUSPEND
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_unsuspend_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Unsuspend Help
	 */
	if(thread->sql.access.qlevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help unsuspend")){
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"unsuspend"));
		send_lang(thread,language,howtosend,snick,nick,"Q_HELP_UNSUSPEND",snick);
		setWebSuccess(thread); /* Indentify webform that changes were applied */
	}
	
	else send_lang(thread,language,howtosend,snick,nick,"Q_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Q SHOWCOMMANDS Command - UNSUSPEND part (/msg Q showcommands)
 * 
 * When the user tries to get general Q help, using /msg Q showcommands, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to<br/>
 * include the information for the UNSUSPEND command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_unsuspend_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"showcommands");
	int Users_Q_Level=(thread->sql.aliases.active>0 ? thread->sql.access.qlevel : is_user);
	/*
	 * Unsuspendchan (Showcommands)
	 */
	if(Users_Q_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"Q_SHOWCOMMANDS_UNSUSPEND",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"unsuspend"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
