/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
/** @file init.c
 *   @brief This is the Module file for initializing all services/modules.
 *
 *   It contains all functions which are neccessary to
 *   load specific services with specific modules.
 */
/*  ************************************************************************************************  */

#include "sysconf.h"
#include "global.h"
#include "modules.h"
#include "services.h"

/**
 * @brief load_modules:  Reads the files in etc / *.modul and loads all files
 *                by using the OrionLoadModule function
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param servicetag Is used to "select" the service for which
 *                 the modules should be loaded
 * @return ModuleHandle
 **/
ModInfo load_modules(Threads *thread, char *servicetag){
	/* Filehandle */
	FILE *in = NULL;
	
	/* ModuleHandle and modulelevels */
	ModInfo ModHandle;
	int User_Level	= 0;
	int Root_Level	= 0;
	
	/* Filename of the module (WITH the Servicestag) */
	char modulefile		[MAX_MODULEFILE_LEN];
	
	/* Filename of the module (WITHOUT the Servicestag) */
	char modulename		[MAX_MODULENAME_LEN];
	
	/* The path to our .conf dir */
	char in_path		[MAX_FILEPATH_LEN];
	
	/* Servicetag + _ */
	char seekservice	[MAX_NICK_LEN];

	bzero(&modulefile,sizeof(modulefile));
	bzero(&modulename,sizeof(modulename));
	bzero(&in_path,sizeof(in_path));
	bzero(&seekservice,sizeof(seekservice));
	
	/* Zero our module handle */
	ModHandle.ModErr = MOD_ERR_NOEXIST;
	ModHandle.module = NULL;
	
	
	/* Look if we should load modules automatically, or if the user decided
	 * to load only a few specific modules */
#ifndef _WIN32
	if(!m->AutoRecognizeModules){
#endif
		snprintf(seekservice,sizeof(seekservice),"%s_",_lc(servicetag));
		snprintf(in_path,sizeof(in_path),"%s/loadmodules.conf",SERVICES_CONF_DIR);
#ifndef _WIN32
	}
	else if(m->AutoRecognizeModules){
		snprintf(seekservice,sizeof(seekservice),"%s_",_lc(servicetag));
		snprintf(in_path,sizeof(in_path),"%s/autoload.conf",SERVICES_CONF_DIR);
	}
#endif
	
	/* Say our owner that we are now loading modules for Service X */
	dlog(thread,0,"  * LOADING modules for service: %s\n",servicetag);
	
	/* We open the config from where we were forced
	 * to read out what modules we have to load */
	in = fopen(in_path,"r");
	
	/* Check if it's really open */
	if(in != NULL){
		while(fgets(modulefile,100,in)){
			
			if(modulefile[0] != '\0' && modulefile[0] != '\n' && modulefile[0] != '#'){

				/* Seek commands for our Service */
				if(strins(modulefile,seekservice)){
                    strdels(modulefile,'n',".c\\");
					strclean(modulefile);
					
					/* Create our modulename - identical to the modulefile, but without the servicetag */
					dynncpy(modulename,modulefile,sizeof(modulename));
					strdels(modulename,'n',seekservice);
					
					/* Now create our ModuleHandle - by loading the modulefile */
					ModHandle.module = NULL;
					ModHandle.ModErr = MOD_ERR_OK;
					ModHandle = OrionLoadModule(thread,modulefile);

					/* Dump Levels */
					User_Level = 0; Root_Level = 0;
				
				    /* Error handling */
					if(ModHandle.ModErr == MOD_ERR_EXISTS){
                        dlog(thread,0,"  * WARNING: [%s] module is already loaded! (Ingore this warning if you are using \"AutoRecognizeModules\")\n",modulefile);
						if(m->AutoRecognizeModules && m->debug>=3){
							dlog(thread,3,"  * HOLD: 3 seconds holding for notification\n");
							sleep(3);
						}
					}
				
					else if(ModHandle.ModErr == MOD_ERR_CANCELLED){
                        dlog(thread,0,"  * WARNING: [%s] module cancelled loading (possibly NOT supported)! (Ingore this warning if you are using \"AutoRecognizeModules\")\n",modulefile);
						if(m->AutoRecognizeModules && m->debug>=3){
							dlog(thread,3,"  * HOLD: 3 seconds holding for notification\n");
							sleep(3);
						}
					}
				
				    /* We had no error so we proceed */
					else if(ModHandle.ModErr == MOD_ERR_OK){
						User_Level = ModHandle.module->LeastUserLevel;
						Root_Level = ModHandle.module->LeastRootLevel;
						domysql(thread,"INSERT INTO `orion_core_commands` (`service`,`modul`,`user_level`,`root_level`) VALUES ('%s','%s',%d,%d)",servicetag,modulename,User_Level,Root_Level);
					}
		
		            /* We had any not specific error - so we tell it our owner and shut down */
					else{
						dlog(thread,0,"  *  ERROR: Loading modulefile %s failed (ErrorNo: %d)\n",modulefile,ModHandle.ModErr);
						dlog(thread,0," [-] Service: %s - ErrorNo(%d): %s\n",servicetag,ModHandle.ModErr,ModuleError(ModHandle.ModErr));
					
						remove_pidfile();
						_orion_exit();
					}
				}
			}
			bzero(&modulefile,sizeof(modulefile));
		}
		
		/* We loaded all modules for Service X and return a message that
		 * the Service with it's modules was successfully loaded */
		printf(" [+] Loaded modules for Service (Servicetag): .... %s\n",servicetag);
		fclose(in);
	}else{
		/* We had an error - weren't able to open the *.conf file (so we shut down) */
		dlog(thread,0,"  *  ERROR: Loading modules for Service: %s\n",servicetag);
		dlog(thread,0," [-] Service: %s - Error: %s\n",servicetag,ModuleError(MOD_ERR_FILE_IO));
		
		remove_pidfile();
		_orion_exit();
	}
	return ModHandle;
}

/*  ************************************************************************************************  */

/**
 * @brief unload_modules:  Reads the loaded modules for target service and unloads them
 *                    by using the OrionUnloadModule function
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param servicetag Is used to "select" the service for which
 *                 the modules should be unloaded
 * @return void
 **/
void unload_modules(Threads *thread, char *servicetag){
	/* Deklarate Variables */
	deepSQL 	commands;
	MOD_ERROR 	ModErr;
	int 		i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&commands,sizeof(commands));

	dlog(thread,0,"  * UNLOADING modules for Service (servicetag): %s\n",servicetag);
	
	commands.res    = doSelectMySQL(thread,"SELECT CONCAT(`service`,'_',`modul`) AS `modulefile` FROM `orion_core_commands` WHERE `service`='%s'",mysql_quote(thread,servicetag));
	commands.fields = mysql_num_rows(commands.res);
	
	for(i=0;i<commands.fields;i++){
		commands.row = mysql_fetch_row(commands.res);
		
		ModErr = OrionUnloadModule(thread,commands.row[0],DO_UNLOAD);
		if(ModErr != MOD_ERR_OK) {
			dlog(thread,0,"  *  ERROR: unloading module %s (servicetag %s)\n",commands.row[0],servicetag);
			dlog(thread,0," [-] Service: %s (tag) - ErrorNo(%d): %s\n",servicetag,ModErr,ModuleError(ModErr));
			
			remove_pidfile();
			_orion_exit();
		}
	}
	mysql_free_result_myr(commands.res);
	
	domysql(thread,"DELETE FROM `orion_core_commands` WHERE `service`='%s'",servicetag);
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief reload_modules:  Reloads all modules within all commands
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param servicetag Is used to "select" the service for which
 *                 the modules should be reloaded
 * @return ModInfo
 **/
ModInfo reload_modules(Threads *thread, char *servicetag){
	/* MySQL Select variable */
	deepSQL commands;

	/* ModuleHandle and modulelevels */
	ModInfo ModHandle;
	int User_Level	= 0;
	int Root_Level	= 0;
	int i			= 0;

	/* The path to our .conf dir */
	char in_path		[MAX_FILEPATH_LEN];

	/* Servicetag + _ */
	char seekservice	[MAX_NICK_LEN];

	bzero(&in_path,sizeof(in_path));
	bzero(&seekservice,sizeof(seekservice));

	/* Zero our module handle */
	ModHandle.ModErr = MOD_ERR_OK;
	ModHandle.module = NULL;
	
	dlog(thread,0,"  * RELOADING modules for service: %s (servicetag)\n",servicetag);
	
	/* UNLOAD all Modules for that Service */
	//---------------------------------------------------------------------------------
	commands.res    = doSelectMySQL(thread,"SELECT CONCAT(`service`,'_',`modul`) AS `modulefile`, `modul` FROM `orion_core_commands` WHERE `service`='%s'",mysql_quote(thread,servicetag));
	commands.fields = mysql_num_rows(commands.res);

	for(i=0;i<commands.fields;i++){
		commands.row = mysql_fetch_row(commands.res);

		ModHandle.ModErr = OrionUnloadModule(thread,commands.row[0],DO_RELOAD);
		if(ModHandle.ModErr != MOD_ERR_OK) {
			dlog(thread,0,"  *  ERROR: unloading module %s (servicetag %s)\n",commands.row[0],servicetag);
			dlog(thread,0," [-] Service: %s (tag) - Error: %s\n",servicetag,ModuleError(ModHandle.ModErr));

			remove_pidfile();
			_orion_exit();
		}
		
		
		/* Now create our ModuleHandle - by loading the modulefile */
		ModHandle = OrionLoadModule(thread,commands.row[0]);

		/* Dump Levels */
		User_Level = 0; Root_Level = 0;

		/* Error handling */
		if(ModHandle.ModErr == MOD_ERR_EXISTS){
            dlog(thread,0,"  * WARNING: [%s] module is already loaded! (Ingore this warning if you are using \"AutoRecognizeModules\")\n",commands.row[0]);
			if(m->AutoRecognizeModules && m->debug>=3){
				dlog(thread,3,"  * HOLD: 3 seconds holding for notification\n");
				sleep(3);
			}
		}

		else if(ModHandle.ModErr == MOD_ERR_CANCELLED){
            dlog(thread,0,"  * WARNING: [%s] module cancelled loading (possibly NOT supported)! (Ingore this warning if you are using \"AutoRecognizeModules\")\n",commands.row[0]);
			if(m->AutoRecognizeModules && m->debug>=3){
				dlog(thread,3,"  * HOLD: 3 seconds holding for notification\n");
				sleep(3);
			}
		}

		/* We had no error so we proceed */
		else if(ModHandle.ModErr == MOD_ERR_OK){
			User_Level = ModHandle.module->LeastUserLevel;
			Root_Level = ModHandle.module->LeastRootLevel;
			domysql(thread,"UPDATE `orion_core_commands` SET `service`='%s',`modul`='%s',`user_level`='%d',`root_level`='%d' WHERE `service`='%s' AND `modul`='%s'",servicetag,commands.row[1],User_Level,Root_Level,servicetag,commands.row[1]);
		}

		/* We had any not specific error - so we tell it our owner and shut down */
		else{
			dlog(thread,0,"  *  ERROR: Loading modulefile %s failed\n",commands.row[0]);
			dlog(thread,0," [-] Service: %s - ErrorNo(%d): %s\n",servicetag,ModHandle.ModErr,ModuleError(ModHandle.ModErr));

			remove_pidfile();
			_orion_exit();
		}
	}
	mysql_free_result_myr(commands.res);

	return ModHandle;
}

#if 0
ModInfo reload_modules(Threads *thread, char *servicetag){
	/* Deklarate Variables */
	deepSQL 	commands;
	MOD_ERROR 	ModErr;
	int 		i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&commands,sizeof(commands));

	/* UNLOAD all Modules for that Service */
	//---------------------------------------------------------------------------------
	dlog(thread,0,"  * UNLOADING modules for Service (servicetag): %s\n",servicetag);

	commands.res    = doSelectMySQL(thread,"SELECT CONCAT(`service`,'_',`modul`) AS `modulefile` FROM `orion_core_commands` WHERE `service`='%s'",mysql_quote(thread,servicetag));
	commands.fields = mysql_num_rows(commands.res);

	for(i=0;i<commands.fields;i++){
		commands.row = mysql_fetch_row(commands.res);

		ModErr = OrionUnloadModule(thread,commands.row[0],DO_RELOAD);
		if(ModErr != MOD_ERR_OK) {
			dlog(thread,0,"  *  ERROR: unloading module %s (servicetag %s)\n",commands.row[0],servicetag);
			dlog(thread,0," [-] Service: %s (tag) - Error: %s\n",servicetag,ModuleError(ModErr));

			remove_pidfile();
			_orion_exit();
		}
	}
	mysql_free_result_myr(commands.res);

	domysql(thread,"DELETE FROM `orion_core_commands` WHERE `service`='%s'",servicetag);
	//---------------------------------------------------------------------------------
	return load_modules(thread, servicetag);
}
#endif
/*  ************************************************************************************************  */
