/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file servers.c
 *   @brief This is the Module file fto handle Servers.
 *
 *   It contains all functions which are neccessary to process and use 
 *   the Servers(onlink on squit) commands.
 *
 *   The Servers File is used to manage Servers all over the Network
 *   (Protocol-Process-File -> PPF)
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "modules.h"
#include "osocket.h"
#include "webform.h"

Servers *servlist 		= NULL;
Servers *me_server 		= NULL;			/* This are we        */
Servers *serv_uplink 	= NULL;    		/* This is our uplink */

char uplink[MAX_SERVER_LEN];
char *TS6UPLINK;
char *TS6SID;

/**
 * @brief findserver_uid:  Find a server by UID, returns NULL if not found
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param s Server struct
 * @param name Server Name
 * @return Server struct
 */
Servers *findserver_uid(Threads *thread, Servers * s, char *name)
{
    Servers *sl;

    if (!name || !*name) {
        return NULL;
    }

    dlog(thread,3,"debug: findserver_uid(%p)\n", name);

    while (s && s->sid && !streqnc(s->sid, name)) {
        if (s->links) {
            sl = findserver_uid(thread, s->links, name);
            if (sl) {
                s = sl;
            } else {
                s = s->next;
            }
        } else {
            s = s->next;
        }
    }
    dlog(thread,3,"debug: findserver_uid(%s) -> %p\n", name, (void *) s);
    return s;
}

/*  ************************************************************************************************  */

/**
 * @brief findserver:  Find a server by name, returns NULL if not found
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param s Server struct
 * @param name Server Name
 * @return Server struct
 */
Servers *findserver(Threads *thread, Servers * s, char *name)
{
    Servers *sl;

    if (!name || !*name) {
        return NULL;
    }

    dlog(thread,3,"debug: findserver(%p)\n", name);

#ifdef USE_THREADS
	my_pthread_mutex_lock(&servers_lock);
#endif
    while (s && !streqnc(s->server, name)) {
        if (s->links) {
            sl = findserver(thread, s->links, name);
            if (sl) {
                s = sl;
            } else {
                s = s->next;
            }
        } else {
            s = s->next;
        }
    }
    dlog(thread,3,"debug: findserver(%s) -> %p\n", name, (void *) s);

#ifdef USE_THREADS
	my_pthread_mutex_unlock(&servers_lock);
#endif
    return s;
}

/*************************************************************************/

/**
 * @brief findserver_next:  Find next server by name, returns NULL if not found
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param s Server struct
 * @param name Server Name
 * @return Server struct
 */
Servers *findserver_next(Threads *thread, Servers * s, char *name){
    Servers *sl;

    if (!name || !*name) {
        return NULL;
    }

    dlog(thread,3,"debug: findserver(%p)\n", name);

#ifdef USE_THREADS
	my_pthread_mutex_lock(&servers_lock);
#endif
    while (s && !streqnc(s->server, name)) {
        if (s->links) {
            sl = findserver_next(thread, s->links, name);
            if (sl) {
                s = sl;
            } else {
                s = s->next;
            }
        } else {
            s = s->next;
        }
    }
    dlog(thread,3,"debug: findserver(%s) -> %p\n", name, (void *) s);
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&servers_lock);
#endif
    return s;
}

/*  ************************************************************************************************  */

/**
 * @brief delete_server:  Deletes a Server from the Servertree
 * 	Remove and free a Server structure. This function is the most complete
 * 	remove treatment a server can get, as it first quits all clients which
 * 	still pretend to be on this server, then it walks through all connected
 * 	servers and disconnects them too. If all mess is cleared, the server
 * 	itself will be too.
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param Server struct
 * @param reason the server quit
 * @return void
 */
static void delete_server(Threads *thread, Servers * serv, char *quitreason){
    Servers *s, *snext;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&servers_lock);
#endif
	
	/* Delete that Server with all users he carries */
	squit(thread,serv->server);

	dlog(thread,4,"SERVER SQUIT: %s\n",serv->server);
	
	dlog(thread,4,"Found %p with name: %s and links to %p\n",serv,serv->server,serv->links);
    s = serv->links;
    dlog(thread,4,"Set s to %p\n",s);
    
	while (s) {
		dlog(thread,4,"While going for %p\n",s);
		snext = s->next;
		dlog(thread,4,"doing recursive for %p (%s)\n",s,s->server);
        
		/* If we perform it from do_squit, we silently delete all linked servers */
		if(quitreason)
			delete_server(thread,s,quitreason);
		
		/* If we perform it from on_squit, we perform it as well for all other linked squitting servers */
		else
			on_squit(thread,s->server);
        	
		s = snext;
	}
    
	dlog(thread,4,"doing if's\n");
	if (serv->prev){
		dlog(thread,4,"if (serv->prev) setting -> serv->prev->next = serv->next; =^ %p = %p\n",serv->prev->next,serv->next);
		serv->prev->next = serv->next;
	}
	if (serv->next){
		dlog(thread,4,"if (serv->next) setting -> serv->next->prev = serv->prev; =^ %p = %p\n",serv->next->prev,serv->prev);
		serv->next->prev = serv->prev;
	}
	if (serv->uplink->links == serv){
		dlog(thread,4,"if (serv->uplink->links == serv) setting -> serv->uplink->links = serv->next; =^ %p = %p\n",serv->uplink->links,serv->next);
		serv->uplink->links = serv->next;
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&servers_lock);
#endif
}


/*  ************************************************************************************************  */

/**
 * @brief do_squit:  Sends out a SQUIT (depending on the Loaded Protocol) - using SQUIT
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param QServer The Server we want to QUIT from the Network
 * @param message The QuitMessage which we want to provide to the public
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_squit(Threads *thread, char *QServer, char *message, ...){
	PROTOCOL_BOOL analyzer;
		
	/* Deklarate Variables */
	va_list va;
	char tmp[BUFFER+1];
	Servers *s;
	
	/* Initialise Variables to (NULL) */	
	bzero(&tmp,sizeof(tmp));

	/* Join reason string */
	va_start	(va,  message);
	vsnprintf	(tmp, sizeof(tmp)-1, message, va);
	va_end		(va);

	/* Analyze Command - is it supported from the IRCd !? */
	analyzer = orion_WRCtranslate_cmd_squit(thread, QServer, tmp);
	if(analyzer >= CMD_NOT_SUPPORTED)
		return analyzer;

	/* Command supported .. we start analyzing it */
    if (IRC.p10protocol || IRC.p10new) {
        s = findserver_uid(thread, servlist, QServer);
        if (!s) {
            s = findserver(thread, servlist, QServer);
        }
    } else {
        s = findserver(thread, servlist, QServer);
    }
    if (!s) {
        dlog(thread,0,"SQUIT for nonexistent server (%s)!!\n", QServer);
        return CMD_ERROR;
    }

	delete_server(thread,s,tmp);
	return analyzer;
}

/*  ************************************************************************************************  */

/**
 * @brief on_squit:  Receives a SQUIT (depending on the Loaded Protocol)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param QServer The Server which sent us a QUIT
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL on_squit(Threads *thread, char *QServer){
	if(QServer == NULL)
		return CMD_ERROR;
		
    Servers *s;
	
    if (IRC.p10protocol || IRC.p10new) {
        s = findserver_uid(thread, servlist, QServer);
        if (!s) {
            s = findserver(thread, servlist, QServer);
        }
    } else {
        s = findserver(thread, servlist, QServer);
    }
    if (!s) {
        dlog(thread,0,"SQUIT for nonexistent server (%s)!!\n", QServer);
        return CMD_ERROR;
    }
		
	/* Inform Moduoles a server quitted */
	_ExecuteEvent(QServer,NULL,"on_squit");
	
	dlog(thread,3,"SQUIT server = %s\n", s->server);
	delete_server(thread,s,NULL);
	return CMD_EXECUTED;
}

/*  ************************************************************************************************  */

/**
 * @brief squit:  Processes the X-SQUIT-Statements for deleting Information from SQL (NO OUTPUT!)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param QServer The Server that sent us a SQUIT (The Server we will delete from the Database)
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL squit(Threads *thread, char *QServer){
	PROTOCOL_BOOL analyzer;
	
	/* Global variables */
	deepSQL squit;
	char chan_update[(MAX_CHANNELNAME_LEN+3)*MAX_CHANS];
	char chan_delete[(MAX_CHANNELNAME_LEN+3)*MAX_CHANS];
	char chan_bot   [(MAX_CHANNELNAME_LEN+3)*MAX_CHANS];
	int i=0;

	/* Initialise Variables to (NULL) */
	bzero(&squit,sizeof(squit));
	bzero(&chan_bot,sizeof(chan_bot));
	bzero(&chan_update,sizeof(chan_update));
	bzero(&chan_delete,sizeof(chan_delete));
	
	dlog(thread,3,"squit(); for server %s\n",QServer);

	domysql(thread,"UPDATE `orion_core_condata` LEFT OUTER JOIN `orion_ns_aliases` ON `orion_ns_aliases`.`nick` = `orion_core_condata`.`nick` SET `active`='0' WHERE `linked`='0' AND `orion_core_condata`.`srv`='%s'",mysql_quote(thread,QServer));
	domysql(thread,"DELETE `orion_core_servers`,`orion_core_condata`,`orion_chan_users`,`orion_ns_aliases` FROM `orion_core_servers` LEFT JOIN `orion_core_condata` ON (`orion_core_servers`.`server`=`orion_core_condata`.`srv`) LEFT OUTER JOIN `orion_chan_users` ON `orion_chan_users`.`nick`=`orion_core_condata`.`nick` LEFT OUTER JOIN `orion_ns_aliases` ON (`orion_ns_aliases`.`nick`=`orion_core_condata`.`nick` AND `orion_ns_aliases`.`linked`=1) WHERE `orion_core_servers`.`server`='%s'",mysql_quote(thread,QServer));
	/** CASCADE (MySQL - InnoDB) does the other statements, because of referencing - chanusers, chanop, ... **/

	DelUserServer(QServer);
	
	/* 	Update our Cache	*/
#ifndef USE_THREADS
	if(streqnc(thread->c.Cdata.server,QServer)){
		bzero(&thread->c.Cdata,sizeof(thread->c.Cdata));
		thread->c.aliases.active = 0;
		
		unknown_access(thread->c.chan.access.targetlevel,sizeof(thread->c.chan.access.targetlevel));
	}else if(streqnc(thread->sql.Cdata.server,QServer)){
		bzero(&thread->sql.Cdata,sizeof(thread->sql.Cdata));
		thread->sql.aliases.active = 0;
		
		unknown_access(thread->c.chan.access.ownlevel,sizeof(thread->c.chan.access.ownlevel));
	}
#endif
	/**			***			**/
	
	squit.res    = doSelectMySQL(thread,"SELECT `orion_cs_core`.`serv`,`orion_cs_core`.`channel`,`orion_cs_core`.`botnick`, SUM(`orion_chan_users`.`nick`=`orion_cs_core`.`botnick`) as `botinchan`, COUNT(`orion_chan_users`.`nick`) as `count_users`,`orion_cs_core`.`settings` FROM `orion_cs_core` LEFT JOIN `orion_chan_users` ON (`orion_cs_core`.`channel` = `orion_chan_users`.`channel`) WHERE `orion_cs_core`.`datecreate`!='' GROUP BY `orion_cs_core`.`channel`");
	squit.fields = mysql_num_rows(squit.res);
		
	for(i=0;i<squit.fields;i++){
		squit.row = mysql_fetch_row(squit.res);
		int settings = stoi(squit.row[5]);

		if(
			(
				( ~settings&CS_INHABIT_BIT && stoi(squit.row[4])==1 && stoi(squit.row[3])==1 ) ||
				( settings&CS_INHABIT_BIT && stoi(squit.row[4])==2 && stoi(squit.row[3])==1)
			) ||
			stoi(squit.row[4])==0
		){
			if(streqnc(squit.row[1],thread->c.chan.name))
				thread->c.chan.countusers--;
			
			if(squit.row[0][0]!='\0'){	// channel empty, but registered by a bot (Q/L/ChanServ/etc...)
			
				straddnc(chan_update,toascii(39),sizeof(chan_update));
				straddns(chan_update,mysql_quote(thread,squit.row[1]),sizeof(chan_update));
				straddnc(chan_update,toascii(39),sizeof(chan_update));
				straddnc(chan_update,',',sizeof(chan_update));
			
				if(squit.row[2]!=NULL && stoi(squit.row[3])==1 && !streqnc(QServer,m->servername) && bs->loaded){
					analyzer = orion_WRCtranslate_cmd_part(thread,squit.row[2],squit.row[1],NULL);
					if(analyzer >= CMD_NOT_SUPPORTED)
						return analyzer;
					
					straddnc(chan_bot,toascii(39),sizeof(chan_bot));
					straddns(chan_bot,mysql_quote(thread,squit.row[1]),sizeof(chan_bot));
					straddnc(chan_bot,toascii(39),sizeof(chan_bot));
					straddnc(chan_bot,',',sizeof(chan_bot));	
				}
				
#if !defined(USE_THREADS) && defined(ENABLE_THREAD_CACHING)
				if(streqnc(squit.row[1],thread->c.chan.name)){
					/*thread->c.chan.datecreate 	= (time_t)0;
					thread->c.chan.limit 		= 0;
					bzero(&thread->c.chan.mode,sizeof(thread->c.chan.mode));
					bzero(&thread->c.chan.link,sizeof(thread->c.chan.link));
					bzero(&thread->c.chan.flood,sizeof(thread->c.chan.flood));
					bzero(&thread->c.chan.key,sizeof(thread->c.chan.key));
			
					if((streqnc(thread->c.chan.serv,"q") && !strcor(thread->c.chan.flags,"t")) || (streqnc(thread->c.chan.serv,"chanserv") && ~thread->c.chan.settings&CS_KEEPTOPIC_BIT))
						bzero(&thread->c.chan.last_topic,sizeof(thread->c.chan.last_topic));*/
					bzero(&thread->c,sizeof(thread->c));
				}
#endif
				
			}else{
				straddnc(chan_delete,toascii(39),sizeof(chan_delete));
				straddns(chan_delete,mysql_quote(thread,squit.row[1]),sizeof(chan_delete));
				straddnc(chan_delete,toascii(39),sizeof(chan_delete));
				straddnc(chan_delete,',',sizeof(chan_delete));
				
#if !defined(USE_THREADS) && defined(ENABLE_THREAD_CACHING)
				if(streqnc(squit.row[1],thread->c.chan.name)){
					bzero(&thread->c,sizeof(thread->c));
				}
#endif
			}
		}
	}
	
	/* Update MySQL (Update channels which are empty & registered, Delete all bans (cauze channel is empty), 
		Delete and the Bot (it was parted in the loop), Delete unregistered Channels that are empty */
		
	if(chan_update[0]!='\0'){
		chan_update[dynlen(chan_update)-1]='\0';
		
		domysql(thread,"UPDATE `orion_cs_core` SET "
			"`datecreate`='',`mode`='',`link`='',`limit`='',`flood`='',`key`=''"
			" WHERE `channel` IN(%s)",chan_update);
		
		domysql(thread,"DELETE `orion_chan_bans`,`orion_chan_expbans` FROM `orion_chan_bans` LEFT OUTER JOIN `orion_chan_expbans` ON `orion_chan_expbans`.`channel`=`orion_chan_bans`.`channel` WHERE `orion_chan_bans`.`channel` IN(%s)",chan_update);
	}
	
	if(chan_delete[0]!='\0'){
		chan_delete[dynlen(chan_delete)-1]='\0';
		domysql(thread,"DELETE FROM `orion_cs_core` WHERE `channel` IN(%s)",chan_delete);
	}
	
	if(chan_bot[0]!='\0'){
		chan_bot[dynlen(chan_bot)-1]='\0';
		domysql(thread,"DELETE `orion_chan_users` FROM `orion_chan_users` LEFT OUTER JOIN `orion_cs_core` ON (`orion_cs_core`.`botnick`=`orion_chan_users`.`nick` AND `orion_cs_core`.`channel`=`orion_chan_users`.`channel`) WHERE `orion_cs_core`.`botnick` IS NOT NULL AND `orion_chan_users`.`channel` IN(%s)",chan_bot);
	}
	
	mysql_free_result_myr(squit.res); //free result
	return CMD_EXECUTED;
}

/*************************************************************************/

/**
 * @brief new_server:  Adds a new server to the servertree (proofes if we are the new server and provides the onCycleGlobal)
 * 	This function makes a new Server structure and links it in the right
 * 	places in the linked list if a Server struct to it's uplink if provided.
 * 	It can also be NULL to indicate it's the uplink and should be first in
 * 	the server list.
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param uplink Server struct
 * @param name Server Name
 * @param desc Server Description
 * @param flags Server Flags, see services.h
 * @param suid Server Universal ID
 * @return Server Struct
 */
Servers *new_server(Threads *thread, Servers * s_uplink, char *name, char *desc, short int flags, char *suid)
{
#ifdef USE_THREADS
	my_pthread_mutex_lock(&servers_lock);
#endif
	Servers *server = (Servers*)malloc(sizeof(Servers));
	bzero(server,sizeof(Servers));
	
    if (!name)
        name = "";
        
	dynncpy(server->server,name,sizeof(server->server));
	dynncpy(server->desc,desc,sizeof(server->desc));
    server->flags = flags;
    server->uplink = s_uplink;
    
	if(suid!=NULL){
		dynncpy(server->sid,suid,sizeof(server->sid));
    } else {
		bzero(&server->sid,sizeof(server->sid));
    }
        
    server->links = NULL;
    server->prev = NULL;

    if (!s_uplink) {
        server->hops = 0;
        server->next = servlist;
        dlog(thread,3,"server->next (%p) == servers (%p)\n",server->next,servlist);
        
        if (servlist)
            servlist->prev = server;
        servlist = server;
    } else {
        server->hops = s_uplink->hops + 1;
        server->next = s_uplink->links;
        dlog(thread,3,"server->next (%p) - uplink->links = (%p) (uplink->links->server = %s)\n",s_uplink,s_uplink->server,s_uplink->links->server);

		if (s_uplink->links)
            s_uplink->links->prev = server;
        s_uplink->links = server;
    }
    /* Check if this is our uplink server */
    if ((s_uplink == me_server) && !(flags & SERVER_JUPED)){
        serv_uplink = server;
        dlog(thread,3,"Uplink set to: %s\n",serv_uplink->server);
	}

    /* Write the StartGlobal (to non-juped servers) */
    if (m->GlobalOnCycleUp[0]!='\0' && streqnc(name,m->servername))
        do_server_notice(thread,m->GlobalOnCycleUp);
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&servers_lock);
#endif
    return server;
}

/*************************************************************************/

/**
 * @brief server:  Introduces a Server to the SQL-Database
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param server The Server we want to introduce
 * @param hopcount The Distance to the added Server
 * @return void
 **/
void server(Threads *thread, char *server, char *uplink, short int hopcount){
	if(!uplink){
        printf("server(); IF adding %s by hop = %d\n",server,hopcount);
		domysql(thread,"INSERT INTO `orion_core_servers` (`server`,`hopcount`) VALUES ('%s','%d')",mysql_quote(thread,server),hopcount);
	}else{
        printf("server(); ELSE adding %s by hop = %d\n",server,hopcount);
		domysql(thread,"INSERT INTO `orion_core_servers` (`server`,`uplink`,`hopcount`) VALUES ('%s','%s','%d')",mysql_quote(thread,server),mysql_quote(thread,uplink),hopcount);
	}
	return;
}

/*************************************************************************/

/**
 * @brief addServer:  Adds a new server
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param source Name of the uplink if any
 * @param servername Name of the server being linked
 * @param hops Number of hops to reach this server
 * @param descript Description of the server
 * @param numeric Server Numberic/SUID
 * @return Servers*
 */
 
//        	addServer(thread, NULL, av[1], av[2], _dstrjoin(' ',av,4), &av[3][occurance]);
Servers* addServer(Threads *thread, char *source, char *servername, char *hops,
               char *descript, char *numeric)
{
    Servers *s;

    if (!source) {
        dlog(thread,3,"debug: Server introduced (%s)\n", servername);
    } else {
        dlog(thread,3,"debug: Server introduced (%s) from %s\n", servername,
        	source);
    }

#ifdef USE_THREADS
	my_pthread_mutex_lock(&servers_lock);
#endif
    if (source == NULL)
        s = me_server;
    else
        s = findserver_next(thread,servlist,source);

    s = new_server(thread, s, servername, descript, 0, numeric);
    server(thread,servername,source,s->hops);
    
    /* We give our selves the Information that we are in the tree */
    if(streq(servername,m->servername))
		me_server = s;
		
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&servers_lock);
#endif
	return s;
}

/*******************************************************************/

/* TS6 UID generator common code.
 *
 * Derived from atheme-services, uid.c (hg 2954:116d46894b4c).
 *         -nenolod
 */
static int ts6_uid_initted = 0;
static char ts6_new_uid[10];    /* allow for \0 */
static int ts6_uid_index = 9;  /* last slot in uid buf */

void ts6_uid_init(void)
{
    /* check just in case... you can never be too safe. */
    if (TS6SID != NULL) {
        snprintf(ts6_new_uid, 10, "%sAAAAAA", TS6SID);
        ts6_uid_initted = 1;
    } else {
        return;
    }
}

void ts6_uid_increment(unsigned int slot)
{
    if (slot != strlen(TS6SID)) {
        if (ts6_new_uid[slot] == 'Z')
            ts6_new_uid[slot] = '0';
        else if (ts6_new_uid[slot] == '9') {
            ts6_new_uid[slot] = 'A';
            ts6_uid_increment(slot - 1);
        } else
            ts6_new_uid[slot]++;
    } else {
        if (ts6_new_uid[slot] == 'Z')
            for (slot = 3; slot < 9; slot++)
                ts6_new_uid[slot] = 'A';
        else
            ts6_new_uid[slot]++;
    }
}

char *ts6_uid_retrieve(void)
{
    if (ts6_uid_initted != 1)
        ts6_uid_init();

    ts6_uid_increment(ts6_uid_index - 1);

    return ts6_new_uid;
}

/* EOF */
