/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file threads.c
 *   @brief This is the Module file for managing Threads.
 *
 *   It contains all functions which are neccessary to
 *   control detached Threads. Or only Thread-Data-Structs. (main-process)
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "services.h"

Threads *threads           = NULL;

/**
 * @brief EndThread:  Kills the given Thread (Closes his logs before ...)
 *
 * @param in_thread The pointer to the Thread from the Thread-struct
 * @return void
 **/
void EndThread(void *in_thread){
	Threads *thread = (Threads *)in_thread;
	close_logs(thread,"\nThread Ended\n");
	KillThread(thread);
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief KillThread:  Deletes the given Thread from the List (DON'T use it directly)
 *               You should use EndThread instead.
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return bool 0 - ERROR (No threaddata deleted), 1- SUCCESS (Deleted target thread's data)
 **/
bool KillThread(Threads *thread){
	
	Threads *ptr = NULL;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&threads_lock);
#endif
	for (ptr = threads; ptr; ptr = ptr->next) {
		if (ptr == thread)
			break;
	}
	if (!ptr){
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&threads_lock);
#endif
		return 0;
	}
	if (thread->prev)
		thread->prev->next = thread->next;
	else
		threads = thread->next;
	if (thread->next)
		thread->next->prev = thread->prev;
		
	free(thread);
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&threads_lock);
#endif
	return 1;
}

/*  ************************************************************************************************  */

/**
 * @brief NewThread:  Starts a new Thread (it will start up
 *               executing the function thread_in
 *
 * @param whatfor The index we want to use with the new-Thread
 * @param func The Function which we want to execute with the new Thread
 * @return void
 **/
void NewThread(char *whatfor, void *(*func) (void *thread_in))
{
#ifdef USE_THREADS
  #ifdef USE_THREADS
	my_pthread_mutex_lock(&threads_lock);
  #endif
	Threads *thread = (Threads*)malloc(sizeof(Threads));
	bzero(thread,sizeof(Threads));
  #ifdef USE_THREADS
	my_pthread_mutex_unlock(&threads_lock);
  #endif

	dyncpy(thread->index,whatfor);

	bzero(&thread->read_netbuf,sizeof(thread->read_netbuf));
	thread->read_curpos = thread->read_netbuf; /* Next byte to return */
	thread->read_bufend = thread->read_netbuf; /* Next position for data from socket */
	thread->read_buftop = thread->read_netbuf + NET_BUFSIZE;
	thread->total_read = 0;
	
	bzero(&thread->write_netbuf,sizeof(thread->write_netbuf));
	thread->write_curpos = thread->write_netbuf;       /* Next byte to write to socket */
	thread->write_bufend = thread->write_netbuf;       /* Next position for data to socket */
	thread->write_buftop = thread->write_netbuf + NET_BUFSIZE;
	
	thread->write_fd = -1;
	thread->lastchar = EOF;
	
	bzero(&thread->inbuf,sizeof(thread->inbuf));
	bzero(&thread->buf,sizeof(thread->buf));
	bzero(&thread->webformOutput,sizeof(thread->webformOutput));
	
  #ifdef USE_THREADS
	/* dynamic.c */
	bzero(&thread->djoin,sizeof(thread->djoin));
	bzero(&thread->dstrjoin,sizeof(thread->dstrjoin));
	bzero(&thread->dstrjoin_det,sizeof(thread->dstrjoin_det));
	
	bzero(&thread->dynadd,sizeof(thread->dynadd));
	
	/* services.c */
	bzero(&thread->time1,sizeof(thread->time1));
	
	/* string management (string.c) */
	bzero(&thread->lc,sizeof(thread->lc));
	bzero(&thread->uc,sizeof(thread->uc));
	
	/* mysql.c */
	bzero(&thread->select,sizeof(thread->select));
	bzero(&thread->aselect,sizeof(thread->aselect));
	bzero(&thread->mysql_sel_var,sizeof(thread->mysql_sel_var));
  #endif
	
	thread->Sock		= 0;
	thread->MSock		= 0;
	thread->Webform		= 0;
	thread->webSuccess	= 0;
	
	thread->QUOTES	= -1;
	
	thread->sqlloghandle	= NULL;
	thread->loghandle		= NULL;
	thread->debughandle		= NULL;
	
	bzero(&thread->sql,sizeof(thread->sql));
	bzero(&thread->c,sizeof(thread->c));

  #ifdef USE_THREADS
	my_pthread_mutex_lock(&threads_lock);
  #endif
	thread->next = threads;

	thread->prev = NULL;
	if (threads)
		threads->prev = thread;
	threads = thread;
  #ifdef USE_THREADS
	my_pthread_mutex_unlock(&threads_lock);
  #endif

	pthread_attr_init(&thread->attr);
	pthread_attr_setdetachstate(&thread->attr, PTHREAD_CREATE_DETACHED);
	pthread_create(&thread->thread, &thread->attr, func, (void *)thread);
#endif
	return;
}


/*  ************************************************************************************************  */

/**
 * @brief RunThread:  Starts a new Thread (it will start up
 *               executing the function thread_in)
 *
 * @param thread The Threads struct
 * @param func The Function which we want to execute with the new Thread
 * @return void
 **/
void RunThread(Threads *thread, void *(*func) (void *thread_in))
{
#ifdef USE_THREADS
	pthread_attr_setdetachstate(&thread->attr, PTHREAD_CREATE_DETACHED);
	pthread_create(&thread->thread, &thread->attr, func, (void *)thread);
#endif
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief NewTData:  Creates a new Thread space (But only the data struct)
 *               NOTE: NO thread will be created! (Used for main process)
 *
 * @param whatfor The index we want to use with the Thread-Data
 * @return void
 **/
Threads *NewTData(char *whatfor)
{
#ifdef USE_THREADS
	my_pthread_mutex_lock(&threads_lock);
#endif
	Threads *thread = (Threads*)malloc(sizeof(Threads));
	bzero(thread,sizeof(Threads));
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&threads_lock);
#endif

	dyncpy(thread->index,whatfor);

	bzero(&thread->read_netbuf,sizeof(thread->read_netbuf));
	thread->read_curpos = thread->read_netbuf; /* Next byte to return */
	thread->read_bufend = thread->read_netbuf; /* Next position for data from socket */
	thread->read_buftop = thread->read_netbuf + NET_BUFSIZE;
	thread->total_read = 0;
	
	bzero(&thread->write_netbuf,sizeof(thread->write_netbuf));
	thread->write_curpos = thread->write_netbuf;       /* Next byte to write to socket */
	thread->write_bufend = thread->write_netbuf;       /* Next position for data to socket */
	thread->write_buftop = thread->write_netbuf + NET_BUFSIZE;
	
	thread->write_fd = -1;
	thread->total_written = 0;
	thread->lastchar = EOF;
	
	bzero(&thread->inbuf,sizeof(thread->inbuf));
	bzero(&thread->buf,sizeof(thread->buf));
	bzero(&thread->webformOutput,sizeof(thread->webformOutput));
	
#ifdef USE_THREADS
	/* dynamic.c */
	bzero(&thread->djoin,sizeof(thread->djoin));
	bzero(&thread->dstrjoin,sizeof(thread->dstrjoin));
	bzero(&thread->dstrjoin_det,sizeof(thread->dstrjoin_det));
	
	bzero(&thread->dynadd,sizeof(thread->dynadd));
	
	/* services.c */
	bzero(&thread->time1,sizeof(thread->time1));
	
	/* string management (string.c) */
	bzero(&thread->lc,sizeof(thread->lc));
	bzero(&thread->uc,sizeof(thread->uc));
	
	/* mysql.c */
	bzero(&thread->select,sizeof(thread->select));
	bzero(&thread->aselect,sizeof(thread->aselect));
	bzero(&thread->mysql_sel_var,sizeof(thread->mysql_sel_var));
#endif
	
	thread->Sock		= 0;
	thread->MSock		= 0;
	thread->Webform		= 0;
	thread->webSuccess	= 0;
	thread->QUOTES		= -1;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&threads_lock);
#endif
	thread->next = threads;

	thread->prev = NULL;
	if (threads)
		threads->prev = thread;
	threads = thread;
	
#ifdef USE_THREADS
	pthread_attr_init(&thread->attr);
	my_pthread_mutex_unlock(&threads_lock);
#endif
	return thread;
}

/*  ************************************************************************************************  */

/**
 * @brief SeekThread:  Searches the Global-Thread-List for a target index
 *               thread with target index
 *
 * @param index The index we want to search for
 * @return Threads* Returns the pointer to the Threads-Data or NULL.
 **/
Threads *SeekThread(char *index){
	Threads *thread = NULL;
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&threads_lock);
#endif
	for (thread = threads; thread; thread = thread->next) {
		if(streqnc(thread->index,index)){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&threads_lock);
#endif
			return thread;
		}
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&threads_lock);
#endif
	return NULL;
}

#ifdef USE_THREADS
/*  ************************************************************************************************  */

/**
 * @brief my_pthread_mutex_unlock:  pthread_mutex_unlock with check for thread compilation
 *
 * @param pthread_mutex_t The mutex we want to lock
 * @return int
 **/
int my_pthread_mutex_unlock(pthread_mutex_t *mutex){
#ifdef USE_THREADS
	if(THREAD_SAFE)
		return pthread_mutex_unlock(mutex);
	else
		return -1;
#endif
	return -1;
}

/*  ************************************************************************************************  */

/**
 * @brief my_pthread_mutex_lock:  pthread_mutex_lock with check for thread compilation
 *
 * @param pthread_mutex_t The mutex we want to lock
 * @return int
 **/
int my_pthread_mutex_lock(pthread_mutex_t *mutex){
#ifdef USE_THREADS
	if(THREAD_SAFE)
		return pthread_mutex_lock(mutex);
	else
		return -1;
#endif
	return -1;
}
#endif

/*  ************************************************************************************************  */

