/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 365                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-11-18 17:38:07 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file users.c
 *   @brief This is the Module file for managing all Global Users.
 *
 *   It contains all functions which are neccessary to
 *   control all Global Users. We manage them in the Database AND
 *   in MEMORY-Lists, since we need the MEMORY Speed in some cases.
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "services.h"

Users *users = NULL;

/**
 * @brief GetUser:  Searches for a Nick in the Global-User-List
 *
 * @param nick The user we search for in our Global-User-List
 * @return Users* Returns the pointer to the list-element where we found "nick"
 **/
Users *GetUser(char *nick){
	Users *user = NULL;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&users_lock);
#endif
	for (user = users; user; user = user->next) {
		if(streqnc(user->nick,nick)){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&users_lock);
#endif
			return user;
		}
	}

#ifdef USE_THREADS
	my_pthread_mutex_unlock(&users_lock);
#endif
	return NULL;
}

/*  ************************************************************************************************  */

/**
 * @brief GetUserByUID:  Searches for a Nick in the Global-User-List - by using the Users ID (UndernetIRCu)
 *
 * @param uid The users UserID for which we want to search
 * @return Users* Returns the pointer to the list-element where we found "nick"
 **/
Users *GetUserByUID(char *uid){
	Users *user = NULL;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&users_lock);
#endif
	for (user = users; user; user = user->next) {
		if(streq(user->uid,uid)){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&users_lock);
#endif
			return user;
		}
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&users_lock);
#endif
	return NULL;
}

/*  ************************************************************************************************  */

/**
 * @brief GetUserChannel:  Searches for a Nick in the Global-User-List - by using the Users ID (UndernetIRCu)
 *
 * @param user The pointer to the User in the Global-User-List, in which's channellist we want to search
 * @param channel The channel we want to find in users' Global-User-List
 * @return Uchannels* Returns the pointer target channel where the user is in
 **/
Uchannels *GetUserChannel(Users *user, char *channel){
	Uchannels *lchannel	= NULL;
	
	if(user==NULL)
		return NULL;
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&users_lock);
#endif
	for (lchannel = user->channels; lchannel; lchannel = lchannel->next) {
		
		if(streqnc(lchannel->channel,channel)){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&users_lock);
#endif
			return lchannel;
		}
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&users_lock);
#endif
	return NULL;
}

/*  ************************************************************************************************  */

/**
 * @brief GetUsersInChannel:  Returns how many users are in a channel
 *
 * @param channel The channel we want to find in users' Global-User-List
 * @return int Returns the number of users for target channel
 **/
int GetUsersInChannel(char *channel){
	Users *user			= NULL;
	Uchannels *lchannel	= NULL;
	
	int counted = 0;

	if(channel==NULL)
		return -1;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&users_lock);
#endif
	for (user = users; user; user = user->next) {
		for (lchannel = user->channels; lchannel; lchannel = lchannel->next) {

			if(streqnc(lchannel->channel,channel) && lchannel->isin_chan==1){
				counted++;
			}
		}
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&users_lock);
#endif
	return counted;
}

/*  ************************************************************************************************  */

/**
 * @brief DelUser:  Deletes the given user from the given channel
 *
 * @param channel The channel from which we want to delete target nick
 * @param nick The nick which we want to delete in target channel
 * @return int 0 - ERROR (No user deletes), 1- SUCCESS (Deleted target user in target channel)
 **/
bool DelUser(char *channel, char *nick){
	Users *user			= NULL;
	Uchannels *lchannel	= NULL;
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&users_lock);
#endif

	for (user = users; user; user = user->next) {	
		if(streqnc(user->nick,nick))
			break;
	}
	
	if(!user){
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&users_lock);
#endif
		return 0;
	}
	
	for (lchannel = user->channels; lchannel; lchannel = lchannel->next) {
		if(streqnc(lchannel->channel,channel))
			break;
	}
	
	if (!lchannel){
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&users_lock);
#endif
		return 0;
	}
		
    if (lchannel->prev)
        lchannel->prev->next = lchannel->next;
            
    else
        user->channels = lchannel->next;
            
    if (lchannel->next)
        lchannel->next->prev = lchannel->prev;
            
    free(lchannel);
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&users_lock);
#endif
	return 1;
}

/*  ************************************************************************************************  */

/**
 * @brief DelUserNick:  Deletes target user from the Global-User-List (DISCONNECT/QUIT)
 *
 * @param nick The nick we want to delete from the list
 * @return int 0 - ERROR (No user deleted), X - SUCCESS (Deleted user from X channels he was in)
 **/
bool DelUserNick(char *nick){
	Users *user			= NULL;
	Uchannels *lchannel	= NULL;
	
	int count = 0;
#ifdef USE_THREADS
	my_pthread_mutex_lock(&users_lock);
#endif
	
	for (user = users; user; user = user->next) {
		if (!user)
			break;
		
		if(streqnc(user->nick,nick))
			break;
	}
	
	if (!user){
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&users_lock);
#endif
		return 0;
	}
	
	//printf("wie stehts mit seinem ersten channel!? = %s\n",user->channels == NULL ? "NULL" : user->channels->channel);
	//printf("wie stehts mit seinem zweiten channel!? = %s\n",user->channels->next == NULL ? "NULL" : user->channels->next->channel);
	//printf("wie stehts mit seinem dritten channel!? = %s\n",user->channels->next->next == NULL ? "NULL" : user->channels->next->next->channel);*/
	for (lchannel = user->channels; lchannel; lchannel = lchannel->next) {
		
		if (!lchannel){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&users_lock);
#endif
			return 0;
		}
				
		if (lchannel->prev)
			lchannel->prev->next = lchannel->next;
				
		else
			user->channels = lchannel->next;
		
		if (lchannel->next)
			lchannel->next->prev = lchannel->prev;


		sfree(lchannel);
		
		count++;
	}
        
    if (user->prev)
        user->prev->next = user->next;
        
    else
        users = user->next;

    if (user->next)
        user->next->prev = user->prev;
        
	sfree(user);
	
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&users_lock);
#endif
	return count;
}

/*  ************************************************************************************************  */

/**
 * @brief DelChannelsForUser:  Deletes all channels of target user (massparted)
 *
 * @param nick The nick for which we want to delete all channel entries
 * @return int 0 - ERROR (No user deleted), X - SUCCESS (Deleted user from X channels he was in)
 **/
bool DelChannelsForUser(char *nick){
	Users *user			= NULL;
	Uchannels *lchannel	= NULL;
	
	int count = 0;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&users_lock);
#endif
	for (user = users; user; user = user->next) {
		if (!user)
			break;
		
		if(streqnc(user->nick,nick))
			break;
	}
	
	if (!user){
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&users_lock);
#endif
		return 0;
	}
		
	for (lchannel = user->channels; lchannel; lchannel = lchannel->next) {
		if (!lchannel){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&users_lock);
#endif
			return 0;
		}
				
		if (lchannel->prev)
			lchannel->prev->next = lchannel->next;
				
		else
			user->channels = lchannel->next;
				
		if (lchannel->next)
			lchannel->next->prev = lchannel->prev;
				
		free(lchannel);
		count++;
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&users_lock);
#endif
	return count;
}

/*  ************************************************************************************************  */

/**
 * @brief DelUserServer:  Deletes all users with their channels from target server (SQUIT)
 *
 * @param server The server from which we want to delete all users
 * @return int 0 - ERROR (No users deleted), X - SUCCESS (Deleted users from X channels)
 **/
bool DelUserServer(char *server){
	Users *user			= NULL;
	Uchannels *lchannel	= NULL;
	
	int count = 0;
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&users_lock);
#endif
	for (user = users; user; user = user->next) {
		if (!user)
			break;
		
		if(streqnc(user->server,server))
			break;
	}
	
	if (!user){
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&users_lock);
#endif
		return 0;
	}

	for (lchannel = user->channels; lchannel; lchannel = lchannel->next) { 
		if (!lchannel){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&users_lock);
#endif
			return 0;
		}
				
		if (lchannel->prev)
			lchannel->prev->next = lchannel->next;
				
		else
			user->channels = lchannel->next;
				
		if (lchannel->next)
			lchannel->next->prev = lchannel->prev;
				
		free(lchannel);
		count++;
	}
        
    if (user->prev)
        user->prev->next = user->next;
        
    else
        users = user->next;
        
    if (user->next)
        user->next->prev = user->prev;
        
    free(user);
	
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&users_lock);
#endif
	return count;
}

/*  ************************************************************************************************  */

/**
 * @brief ChangeUser:  Changes a users nick in the Global-User-List
 *
 * @param oldnick The nick which we want to change
 * @param newnick The the nick we want to set to "oldnick"
 * @return Users* Returns the pointer from the changed user-element
 **/
Users *ChangeUser(char *oldnick, char *newnick){
	Users *user = NULL;
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&users_lock);
#endif
	for (user = users; user; user = user->next) {
		if(!user){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&users_lock);
#endif
			return NULL;
		}
		
		if(streqnc(user->nick,oldnick)){
			bzero(&user->nick,sizeof(user->nick));
			dynncpy(user->nick,newnick,sizeof(user->nick));
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&users_lock);
#endif
			return user;
		}
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&users_lock);
#endif
	return NULL;
}

/*  ************************************************************************************************  */

/**
 * @brief addUser:  Adds a users nick to the Global-User-List
 *
 * @param nick The nick which we want to add
 * @param server The server on which the "nick" connected
 * @return Users* Returns the pointer from the added user-element
 **/
Users *addUser(char *nick, char *server){
#ifdef USE_THREADS
	my_pthread_mutex_lock(&users_lock);
#endif

	Users *user = (Users*)malloc(sizeof(Users));
	bzero(user,sizeof(Users));

	dynncpy(user->nick,nick,MAX_NICK_LEN);
	dynncpy(user->server,server,MAX_SERVER_LEN);
	
	bzero(&user->vident,sizeof(user->vident));
	bzero(&user->ident,sizeof(user->ident));
	bzero(&user->vhost,sizeof(user->vhost));
	bzero(&user->host,sizeof(user->host));
	bzero(&user->ip,sizeof(user->ip));
	bzero(&user->lastcommand,sizeof(user->lastcommand));
	user->time 			  = 0;
	user->faketype 		  = 0;
	
	user->LastJFloodCheck = 0;
	user->LastPFloodCheck = 0;
	user->LastNFloodCheck = 0;
	user->LastServicesMsg = 0;
	user->LastNSRegister  = 0;
	
	user->Ignoretime	  = (time_t)0;
	user->ns_warntime	  = (time_t)0;
	user->rights		  = is_user;
	user->settings		  = 0;
	user->registered      = 0;
	
	user->JoinFloodLines   = 0;
	user->PartFloodLines   = 0;
	user->NickFloodLines   = 0;
	user->ServicesMsgLines = 0;
	user->NSRegisterMsgs   = 0;
	
	user->channels = NULL;

	user->next = users;
	user->prev = NULL;
	if (users)
		users->prev = user;
	users = user;
	
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&users_lock);
#endif
	return users;
}

/*  ************************************************************************************************  */

/**
 * @brief addUserChannel:  Adds the given User-element a channel
 *
 * @param user The pointer for the user-element where we want to add target channel
 * @param channel The channel which we want to add for target user
 * @return Uchannels* Returns the pointer of the added channel-element
 **/
Uchannels *addUserChannel(Users *user, char *channel){

    Uchannels *lchannel = GetUserChannel(user,channel);
	if(lchannel!=NULL){
#ifdef USE_THREADS
		my_pthread_mutex_lock(&users_lock);
#endif
		lchannel->isin_chan			= 1;
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&users_lock);
#endif
	    return lchannel;
	}

#ifdef USE_THREADS
	my_pthread_mutex_lock(&users_lock);
#endif
	lchannel = (Uchannels*)malloc(sizeof(Uchannels));
	bzero(lchannel,sizeof(Uchannels));

	dynncpy(lchannel->channel,channel,MAX_CHANNELNAME_LEN);
	bzero(&lchannel->lastmsg,sizeof(lchannel->lastmsg));
	
	lchannel->Repeat		 = 0;
	lchannel->LastFloodCheck = (time_t)0;
	lchannel->FloodLines	 = 0;
	
	lchannel->kicked_repeat		= 0;
	lchannel->kicked_caps		= 0;
	lchannel->kicked_flood		= 0;

	lchannel->kicked_colors		= 0;
	lchannel->kicked_bolds		= 0;
	lchannel->kicked_underlines	= 0;
	lchannel->kicked_reverses	= 0;
	lchannel->kicked_badwords	= 0;
	
	lchannel->isin_chan			= 1;
	
	lchannel->next = user->channels;
    lchannel->prev = NULL;
    if (user->channels)
        user->channels->prev = lchannel;
        
    user->channels = lchannel;
    
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&users_lock);
#endif
	return lchannel;
}

/*  ************************************************************************************************  */

/**
 * @brief PartChannel:  Updates the structure; so we know user isn't in channel X anymore (but we keep it's data)
 *
 * @param user The pointer to the User in the Global-User-List, in which's channellist we want to search
 * @param channel The channel we want to find in users' Global-User-List
 * @return Uchannels* Returns the pointer target channel where the user is in
 **/
Uchannels *PartChannel(Users *user, char *channel){
	Uchannels *lchannel	= NULL;

	if(user==NULL)
		return NULL;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&users_lock);
#endif
	for (lchannel = user->channels; lchannel; lchannel = lchannel->next) {

		if(streqnc(lchannel->channel,channel)){
			lchannel->isin_chan = 0;
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&users_lock);
#endif
			return lchannel;
		}
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&users_lock);
#endif
	return NULL;
}

/*  ************************************************************************************************  */

/**
 * @brief PartChannelNick:  Updates the structure; so we know user isn't in channel X anymore (but we keep it's data)
 *
 * @param nick The nickname which we want to part from channel
 * @param channel The channel we want to find in users' Global-User-List
 * @return Uchannels* Returns the pointer target channel where the user is in
 **/
Uchannels *PartChannelNick(char *nick, char *channel){
	Users *user 		= GetUser(nick);
	Uchannels *lchannel	= NULL;

	if(user==NULL)
		return NULL;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&users_lock);
#endif
	for (lchannel = user->channels; lchannel; lchannel = lchannel->next) {

		if(streqnc(lchannel->channel,channel)){
			lchannel->isin_chan = 0;
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&users_lock);
#endif
			return lchannel;
		}
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&users_lock);
#endif
	return NULL;
}

/*  ************************************************************************************************  */

/**
 * @brief PartAllChannels:  Searches for a Nick in the Global-User-List - by using the Users ID (UndernetIRCu)
 *
 * @param user The pointer to the User in the Global-User-List, in which's channellist we want to search
 * @return void
 **/
void PartAllChannels(Users *user){
	Uchannels *lchannel	= NULL;

	if(user==NULL)
		return;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&users_lock);
#endif
	for (lchannel = user->channels; lchannel; lchannel = lchannel->next) {
		lchannel->isin_chan = 0;
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&users_lock);
#endif
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief addUserChannelMore:  Adds the given User-element to a channel
 *                       (with his Last-Message)
 *
 * @param user The pointer for the user-element where we want to add target channel
 * @param channel The channel which we want to add for target user
 * @param lastmsg The lastmessage sent from target user in target channel (BotServ REPEAT)
 * @return Uchannels* Returns the pointer of the added channel-element
 **/
/*Uchannels *addUserChannelMore(Users *user, char *channel, char *lastmsg){
#ifdef USE_THREADS
	my_pthread_mutex_lock(&users_lock);
#endif
	Uchannels *lchannel = (Uchannels*)malloc(sizeof(Uchannels));
	bzero(lchannel,sizeof(Uchannels));

	dynncpy(lchannel->channel,channel,MAX_CHANNELNAME_LEN);
	dynncpy(lchannel->lastmsg,lastmsg,sizeof(lchannel->lastmsg));
	
	lchannel->Repeat		 = 0;
	lchannel->LastFloodCheck = (time_t)0;
	lchannel->FloodLines	 = 0;
	
	lchannel->kicked_repeat		= 0;
	lchannel->kicked_caps		= 0;
	lchannel->kicked_flood		= 0;

	lchannel->kicked_colors		= 0;
	lchannel->kicked_bolds		= 0;
	lchannel->kicked_underlines	= 0;
	lchannel->kicked_reverses	= 0;
	lchannel->kicked_badwords	= 0;
	
	lchannel->next = user->channels;
    lchannel->prev = NULL;
    if (user->channels)
        user->channels->prev = lchannel;
        
    user->channels = lchannel;

#ifdef USE_THREADS
	my_pthread_mutex_unlock(&users_lock);
#endif
	return lchannel;
}*/

/*  ************************************************************************************************  */

/**
 * @brief GetUserIntID:  Converts the string-UID to an integer
 *
 * @param id The string-UID
 * @return long Returns the long of given string-UID
 **/
int GetUserIntID(char *id){
	int i = 0;
	int a = 0;
	
	size_t b = dynlen(id)-1;
	//printf("b = %d\n",b);
	
	for(i=b;i>-1;i--){
		a+=(int)id[i]+(abs(i-b)*26);
	}
	return a;
}

/*  ************************************************************************************************  */

/**
 * @brief GetUserLastUID:  Searches for the last UID on a given Server (SID)
 *
 * @param sid The ServerID where we want to get the laszt UID from
 * @return lastid Returns the last used UID
 **/
char *GetUserLastUID(char *sid){
	Users *user = NULL;
	char *lastid = NULL;
	int lastuid = 0;
	int tempid = 0;
	
	if(IRC.p10protocol)
		lastuid = 41611;
	else
		lastuid = 170435210;
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&users_lock);
#endif
	for (user = users; user; user = user->next) {
		
		if(IRC.p10protocol){
			if(user->uid[0]==sid[0] && user->uid[1]==sid[1]){
			
				tempid=base64dec(&user->uid[2]);
				if(tempid>=lastuid){
					lastuid=tempid;
					lastid=&user->uid[2];
				}
			}
		}
		else{
			if(user->uid[0]==sid[0] && user->uid[1]==sid[1] && user->uid[2]==sid[2]){
			
				tempid=base64dec(&user->uid[3]);
				if(tempid>=lastuid){
					lastuid=tempid;
					lastid=&user->uid[3];
				}
			}
			
		}
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&users_lock);
#endif
	
	if(IRC.p10protocol && lastuid==41610)
		return "AAB";
	
	if(lastuid==170435210)
		return "AAAAA";
	
	return lastid;
}

/*  ************************************************************************************************  */

/**
 * @brief GetNewID:  Creates a new UID from the last given UID
 *
 * @param id The last used UserID
 * @return *uid Returns the the new UID we should use
 **/
char *GetNewID(char *id){
	static char uid[10];
	bzero(&uid,sizeof(uid));
	
	size_t b 	= dynlen(id)-1;
	int i 		= 0;
	int done 	= 0;
	uid[b]='\0';
	//printf("b = %d\n",b);
	
	for(i=b;i>-1;i--){
		//printf("looping i = %d\n",i);
		if((int)id[i]<90 && !done){
			//printf("adding char %c\n",(char)((int)id[i]+1));
			uid[i]=(char)((int)id[i]+1);
			done=1;
		}
		else if((int)id[i]==90 && !done){
			if((int)id[0]<90)
				uid[i]='A';
		}
		else{
			//printf("adding char in else %c\n",id[i]);
			uid[i]=id[i];
		}
	}
	uid[b+1]='\0';
	//printf("Built UID = %s\n",uid);
	return uid;
}

/*  ************************************************************************************************  */
